#------------------------------------------------------------------------------
# gpWidgets 1.0
#
# General Purpose Widget Utilities for snackAmp Player in Tcl/Tk
#
# Copyright (C) 2001 Tom Wilkason
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# Please send any comments/bug reports to
# tom.wilkason@cox.net  (Tom Wilkason)
#
#------------------------------------------------------------------------------
set [file rootname [file tail [info script]]]_History {
   $Header: /cvsroot/snackamp/snackamp/lib/gpWidgets.tcl,v 1.60 2005/06/05 01:58:23 wilkason Exp $
}
package require BWidget
#//////////////////////////////////////////////////////////////////////////////
#                   B A L L O O N    H E L P
#//////////////////////////////////////////////////////////////////////////////
namespace eval balloon {
   variable IDplayListCursor 0
   variable playListCursor   0
}
#------------------------------------------------------------------------------
# Function   : balloon
# Description: Bind balloon help to some widget
# Author     : Tom Wilkason
# Date       : 1/2/2000
#------------------------------------------------------------------------------
proc balloon::button {w help} {
   bind $w <Any-Enter> [list balloon::_buttonProcess enter %W %X %Y $help]
   bind $w <Any-Leave> [list balloon::_buttonProcess leave %W %X %Y $help]
   bind $w <ButtonPress-1> [list balloon::_buttonProcess leave %W %X %Y $help]
   #bind $w <Any-Motion> [list balloon::Destroy %W.balloon]
}
#------------------------------------------------------------------------------
# Process button relief and button help
#------------------------------------------------------------------------------
proc balloon::_buttonProcess {how W X Y help} {
   if {$how=="enter"} {
      balloon::afterShow $W.balloon [list balloon::show $W $X $Y $help]
      balloon::_bubbleIt $W 1
   } else {
      balloon::Destroy $W.balloon
      balloon::_bubbleIt $W 0
   }
}
#------------------------------------------------------------------------------
# Function   : balloon
# Description: Bind some widget to balloon help
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc balloon::define {w help} {
   bind $w <Any-Enter> [list balloon::afterShow %W.balloon [list balloon::show %W %X %Y $help]]
   bind $w <Any-Leave> [list balloon::Destroy %W.balloon]
}

#------------------------------------------------------------------------------
# Function   : bubbleIt
# Description: Raise the bubble on the button if it is not depressed (like pause)
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc balloon::_bubbleIt {W Enter} {
   if {[$W cget -relief] != "sunken"} {
      if {$Enter} {
         $W configure -relief raised
      } else {
         $W configure -relief flat
      }
   }
}
#------------------------------------------------------------------------------
# Destroy a widget and cancel any pending destroys
#------------------------------------------------------------------------------
proc balloon::Destroy {widget} {
   destroy $widget
   after cancel [list destroy $widget]
}
#------------------------------------------------------------------------------
# Schedule a balloon to be shown in the future
#------------------------------------------------------------------------------
proc balloon::afterShow {widget cmd} {
   destroy $widget
   after cancel [list destroy $widget]
   after $::snackAmpSettings(balloonDuration) [list destroy $widget]
   return [eval [list after $::snackAmpSettings(balloonDelay) $cmd]]
}
#------------------------------------------------------------------------------
# Function   : balloon::plDefine
# Description: Bind balloon details on a song when moving mouse over it
#              for the play list box
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc balloon::plDefine {args} {
   package require snID3
   variable IDplayListCursor 0
   variable playListCursor 0
   bind [LB::body] <Any-Motion> {
      if {$::snackAmpSettings(ID3ballon)} {
         balloon::_PlayListName %W [LB::Y %y]
         balloon::Destroy %W.balloon
         after cancel $balloon::IDplayListCursor
         set balloon::IDplayListCursor [balloon::afterShow %W.balloon [list balloonSongInfo %W $balloon::playListCursor]]
      }
   }
   bind [LB::body] <Any-Leave> [list balloon::Destroy %W.balloon]
}
#------------------------------------------------------------------------------
# balloon::widgetDefine
# Balloon popup the current playing track info when the mouse is over the
# passed in widget.
#------------------------------------------------------------------------------
proc balloon::widgetDefine {W} {
   package require snID3
   bind $W <Any-Enter> {
      balloon::afterShow %W.balloon [list balloonSongInfo %W $::TA(currentMF)]
   }
   bind $W <Any-Leave>     {destroy %W.balloon}
   bind $W <ButtonPress-1> {destroy %W.balloon}
   bind $W <Any-Motion>    {destroy %W.balloon}
}
#------------------------------------------------------------------------------
# Function   : balloonSongInfo
# Description: Pop-up ballon help on a selected track, get data from database
# Author     : Tom Wilkason
# Date       : 11/4/2001
#------------------------------------------------------------------------------
proc balloonSongInfo {W fullName} {
   if {[set fullName $fullName]!={}} {
      balloon::show $W [winfo pointerx .] [winfo pointery .] [catalog::trackTextData $fullName Details]
   }
}
#------------------------------------------------------------------------------
# Function   : playListName
# Description: Save away the song name for the song under the mouse cursor
#             in the main playlist
# Author     : Tom Wilkason
# Date       : 11/11/2001
#------------------------------------------------------------------------------
proc balloon::_PlayListName {W y} {
   global LB
   variable playListCursor
   set closest [$LB nearest $y]
   if {[catch {
      set playListCursor [LB::keyToName [$LB get $closest]]
   }]} {
      set playListCursor {}
   }
}
#------------------------------------------------------------------------------
# Function   : balloon::show
# Description: Pop-up the ballon help based on a callback for the widget enter
# Author     : Tom Wilkason
# Date       : 1/2/2000
#------------------------------------------------------------------------------
proc balloon::show {w x y arg} {
   # Don't bother if the mouse has left the widget
   # The other approach is the track afters and cancel
   # them on leaving, this is easier.
   if {[eval winfo containing  [winfo pointerxy .]]!=$w} {
      return
   }
   set top $w.balloon
   destroy $top
   toplevel $top -bd 1 -bg black -cursor $::TA(arrow)  -class SnackAmp
   wm overrideredirect $top 1
   pack [message $top.txt -aspect 1000 -bd 2 -bg lightyellow -fg black -font balloon -text $arg]
   # if 0 for pointer x/y issued, then figure out proper location
   # Use current pointer always, not what was passed in in after call
   set x [winfo pointerx $w]
   set y [winfo pointery $w]
   set tWidth [winfo reqwidth $top.txt]
   set tHeight [winfo reqheight $top.txt]
   # Keep the balloon in the visible screen, flip as necessary
   # Don't colocate with pointer location otherwise
   # flashing will occur (bug?)
   if {($x+$tWidth) > $::TA(screen_right)} {
      set x [expr {$x-$tWidth-1}]
   } else {
      incr y
   }
   if {($y+$tHeight) > $::TA(screen_bottom)} {
      set y [expr {$y-$tHeight-1}]
   } else {
      incr x
   }
   # Offset by one pixel so balloon is off cursor
   wm geometry $top ${tWidth}x${tHeight}+$x+$y
   fade::off $top
   raise $top
   update

   # Force on top if possible
   ontop::ifpossible $w $top
   fade::in $top
}
#------------------------------------------------------------------------------
# Function   : Scrolled_Listbox
# Description: Utility function to create a scrolled listbox
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc Scrolled_Listbox { f args } {
   frame $f
   listbox $f.list \
      -xscrollcommand [list scrollCmd $f.xscroll  [list grid $f.xscroll -row 1 -column 0 -sticky ew]] \
      -yscrollcommand [list scrollCmd $f.yscroll  [list grid $f.yscroll -row 0 -column 1 -sticky ns]]
   eval {$f.list configure} $args
   scrollbar $f.xscroll -orient horizontal -command [list $f.list xview]
   scrollbar $f.yscroll -orient vertical   -command [list $f.list yview]
   grid $f.list -sticky news
   ##
   # Only the data should expand, not the scroll bars
   #
   grid rowconfigure    $f 0 -weight 1
   grid columnconfigure $f 0 -weight 1
   ##
   # Modify to not resize when the scroll bars go away (make a minimum size)
   #
   grid columnconfigure $f 1 -minsize 0 ;#1
   grid rowconfigure    $f 1 -minsize 0 ;#1

   # For *nix mousewheels
   mscroll $f.list

   return $f.list
}
##
# mclistbox inside a scrolled window
#
proc Scrolled_mcListbox { f args } {
   package require mclistbox
   frame $f
   # Create the listbox
   ::mclistbox::mclistbox $f.list \
         -xscrollcommand [list scrollCmd $f.xscroll  [list grid $f.xscroll -row 1 -column 0 -sticky ew]] \
         -yscrollcommand [list scrollCmd $f.yscroll  [list grid $f.yscroll -row 0 -column 1 -sticky ns]]

   eval {$f.list configure} $args
   scrollbar $f.xscroll -orient horizontal -command [list $f.list xview]
   scrollbar $f.yscroll -orient vertical   -command [list $f.list yview]
   grid $f.list -sticky news
   ##
   # Only the data should expand, not the scroll bars
   #
   grid rowconfigure    $f 0 -weight 1
   grid columnconfigure $f 0 -weight 1
   ##
   # Modify to not resize when the scroll bars go away (make a minimum size)
   #
   grid columnconfigure $f 1 -minsize 0 ;#1
   grid rowconfigure    $f 1 -minsize 0 ;#1

   # For *nix mousewheels
   mscroll $f.list

   return $f.list
}
#------------------------------------------------------------------------------
# Set the highlight row to inverse video of the other rows
#------------------------------------------------------------------------------
proc lbFlipColor {w} {
   set sfg [$w cget -background]
   set sbg [$w cget -foreground]
   $w configure -selectbackground $sbg -selectforeground $sfg
}
##
# mclistbox inside a scrolled window
#
proc Scrolled_tlListbox { f args } {
   package require mclistbox
   frame $f
   # Create the listbox
   tablelist::tablelist $f.list \
         -xscrollcommand [list scrollCmd $f.xscroll  [list grid $f.xscroll -row 1 -column 0 -sticky ew]] \
         -yscrollcommand [list scrollCmd $f.yscroll  [list grid $f.yscroll -row 0 -column 1 -sticky ns]] \

   eval {$f.list configure} $args
   scrollbar $f.xscroll -orient horizontal -command [list $f.list xview]
   scrollbar $f.yscroll -orient vertical   -command [list $f.list yview]
   grid $f.list -sticky news
   ##
   # Only the data should expand, not the scroll bars
   #
   grid rowconfigure    $f 0 -weight 1
   grid columnconfigure $f 0 -weight 1
   ##
   # Modify to not resize when the scroll bars go away (make a minimum size)
   #
   grid columnconfigure $f 1 -minsize 0 ;#1
   grid rowconfigure    $f 1 -minsize 0 ;#1

   # For *nix mousewheels
   mscroll $f.list

   return $f.list
}
#------------------------------------------------------------------------------
# Binds the *nix mousewheel events to scroll a scrollbar
#------------------------------------------------------------------------------
proc mscroll {bindtag} {
   if {$::tcl_platform(platform) == "unix"} {
      bind $bindtag <Button-5> {%W yview scroll 5 units;break}
      bind $bindtag <Button-4> {%W yview scroll -5 units;break}
      bind $bindtag <Shift-Button-5> [list %W yview scroll 1 units]
      bind $bindtag <Shift-Button-4> [list %W yview scroll -1 units]
      bind $bindtag <Control-Button-5> [list %W yview scroll 1 pages]
      bind $bindtag <Control-Button-4> [list %W yview scroll -1 pages]
   }
}
# Update for sticky pop-up
# This is part of the 8.5 head
if {($::tcl_platform(platform) == "unix") && ($::tcl_version < 8.5)} {
   proc ::tk::MenuInvoke {w buttonRelease} {
      variable ::tk::Priv

      if {$buttonRelease && [string equal $Priv(window) {}]} {
         # Mouse was pressed over a menu without a menu button, then
         # dragged off the menu (possibly with a cascade posted) and
         # released.  Unpost everything and quit.

         $w postcascade none
         $w activate none
         event generate $w <<MenuSelect>>
         MenuUnpost $w
         return
      }
      if {[string equal [$w type active] "cascade"]} {
         $w postcascade active
         set menu [$w entrycget active -menu]
         MenuFirstEntry $menu
      } elseif {[string equal [$w type active] "tearoff"]} {
         ::tk::TearOffMenu $w
         MenuUnpost $w
      } elseif {[string equal [$w cget -type] "menubar"]} {
         $w postcascade none
         set active [$w index active]
         set isCascade [string equal [$w type $active] "cascade"]

         # Only de-activate the active item if it's a cascade; this prevents
         # the annoying "activation flicker" you otherwise get with
         # checkbuttons/commands/etc. on menubars

         if { $isCascade } {
            $w activate none
            event generate $w <<MenuSelect>>
         }

         MenuUnpost $w

         # If the active item is not a cascade, invoke it.  This enables
         # the use of checkbuttons/commands/etc. on menubars (which is legal,
         # but not recommended)

         if { !$isCascade } {
            uplevel #0 [list $w invoke $active]
         }
      } else {
         set active [$w index active]
         if {$Priv(popup) eq "" || $active ne "none"} {
            MenuUnpost $w
         }
         uplevel #0 [list $w invoke active]
      }
   }


}

#------------------------------------------------------------------------------
# Function   : getEntryDialog
# Description: Generic dialog to ask user for some text
# Author     : Tom Wilkason
# Date       : 3/12/2001
#------------------------------------------------------------------------------
proc getEntryDialog {message _variable title} {
   set i 0
   upvar $_variable variable
   while {[info command .ged$i] != {}} {incr i}
   set T .ged$i
   toplevel $T
   wm title $T $title
   set ::TA(dialogEntry) $variable
   wm protocol $T WM_DELETE_WINDOW {set ::TA(continue) 1}
   # wm overrideredirect $T 1
   set len [expr {round([string length $variable]*1.25)}]
   if {$len < 20} {
      set len 20
   }
   pack [label $T.l -text $message -anchor w] \
      -side top -fill x -anchor w
   pack [entry $T.e -width $len \
      -textvariable ::TA(dialogEntry) -exportselection false] \
      -side top -fill x
   $T.e config -background $::snackAmpSettings(ebg) -foreground $::snackAmpSettings(efg)
   settings::regColor $T.e -background ::snackAmpSettings(ebg) -foreground ::snackAmpSettings(efg)
   focus $T.e
   pack [frame $T.s -height 2 -borderwidth 2 -relief sunken] -side top -fill x -pady 3
   pack [frame $T.f] -side top -fill x -expand true
   pack [button $T.f.o -padx 4 -text OK -compound left -image saImages::Apply \
      -command [list set ::TA(continue) ok] \
      -default active -relief raised] -side left
   pack [button $T.f.c -padx 4 -text Cancel -compound left -image saImages::Close \
      -command [list set ::TA(continue) cancel] \
      -default normal -borderwidth 1 -relief raised] -side right
   bind $T <Return> [list $T.f.o invoke]
   bind $T <Escape> [list $T.f.c invoke]
   grab set $T
   catch {wm transient $T [winfo toplevel [focus]]}
   centerWindow $T
   tkwait visibility $T
   focus $T
   set ::TA(continue) 0
   myIcon $T

   #-------WAIT---------
   vwait ::TA(continue)
   grab release $T
   destroy $T
   set variable $::TA(dialogEntry)
   return $::TA(continue)
}
#------------------------------------------------------------------------------
# Function   : myIcon
# Description: Try to place an icon for this window
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc myIcon {w} {
   catch {
      switch -- $::tcl_platform(platform) {
         "unix" {
            #8.4.8 needed for iconphoto
            if {[catch {wm iconphoto $w saImages::saLogo} result] } then {
               wm iconbitmape $w snackPause
            }
         }
         "windows" {winico_seticon $w $::TA(IconFile)}
         default {}
      }
   } result
}
#------------------------------------------------------------------------------
# Function   : wmIconImage
# Description: This only works from some window managers
#              Not currently used
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc wmIconImage {toplevel image} {
    variable _imgIcon_count
    if {![info exists _imgIcon_count]} {
        set _imgIcon_count 0
    }
    set iconTop [toplevel ._iconImage[incr _imgIcon_count]]
    pack [label $iconTop.l -image $image]
    wm iconwindow $toplevel $iconTop
}
#------------------------------------------------------------------------------
# Copyright (C) 1998 D. Richard Hipp
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   drh@acm.org
#   http://www.hwaci.com/drh/
#------------------------------------------------------------------------------
proc labeledFrame {w args} {
   frame $w -bd 0
   label $w.l
   frame $w.f -bd 2 -relief groove
   frame $w.f.f
   pack $w.f.f
   set text {}
   set font {}
   set padx 3
   set pady 7
   set ipadx 2
   set ipady 9
   foreach {tag value} $args {
      switch -- $tag {
         -font  {set font $value}
         -text  {set text $value}
         -padx  {set padx $value}
         -pady  {set pady $value}
         -ipadx {set ipadx $value}
         -ipady {set ipady $value}
         -bd     {$w.f config -bd $value}
         -relief {$w.f config -relief $value}
      }
   }
   if {[string length $font]} {
      $w.l config -font $font
   }
   $w.l config -text $text
   pack $w.f -padx $padx -pady $pady -fill both -expand 1
   place $w.l -x [expr {$padx+10}] -y $pady -anchor w
   pack $w.f.f -padx $ipadx -pady $ipady -fill both -expand 1
   raise $w.l
   return $w.f.f
}
#------------------------------------------------------------------------------
# Function   : scrollCmd
# Description: Generic callback to show/hide scrollbars when needed
#              scroll bars if they aren't needed
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc scrollCmd {scrollbar geoCmd offset size} {
   catch {
      if {$offset > 0.02 || $size != 1.0} {
         eval $geoCmd
         $scrollbar set $offset $size
      } else {
         set manager [lindex $geoCmd 0]
         $manager forget $scrollbar
      }
   }
}
namespace eval tabbedOptionFrame {
   variable TOF
}
#------------------------------------------------------------------------------
# Function   : tabbedOptionFrame::create
# Description: Create a tabbed option frame
# Author     : Tom Wilkason
# Date       : 5/15/2002
#------------------------------------------------------------------------------
proc tabbedOptionFrame::Create {base args} {
   variable TOF
   set myArgs [list]
   set options [list]
   set help [list]
   set height 20
   set width 20
   foreach {config setting} $args {
      switch -glob -- $config {
         -opt* {set options $setting}
         -help* {set help $setting}
         -Width* {set width $setting}
         -Height* {set height $setting}
         -tit* {}
         default {lappend myArgs $config $setting}
      }
   }
   eval frame {$base} $myArgs
   ##
   # Listbox, register snackAmp colors. Contains the options
   # allowing user to select them
   #
   set plb [Scrolled_Listbox $base.lb -exportselection no \
      -height $height -width $width \
      -foreground       $::snackAmpSettings(fg)        \
      -background       $::snackAmpSettings(bg)        \
      -selectforeground $::snackAmpSettings(bg)        \
      -selectbackground $::snackAmpSettings(fg) \
      -font listbox]

   settings::regColor $plb  \
      -foreground       ::snackAmpSettings(fg) \
      -background       ::snackAmpSettings(bg) \
      -selectforeground ::snackAmpSettings(bg) \
      -selectbackground ::snackAmpSettings(fg)
   ##
   # For each option passed in with -options, insert it into the listbox
   # and bind the select on the listbox to show that frame
   #
   set i 0
   foreach {opt} $options {
      set hull [labeledFrame $base.fr$i -text $opt -bd 2 -relief groove -font listbox]
      set TOF($plb,$i,opt) $opt
      set TOF($plb,$i,frame) $base.fr$i
      set TOF($plb,$i,hull) $hull
      incr i
      $plb insert end $opt
   }
   # Insert the options
   # When user selects an option, pack it
   bind $plb <ButtonPress-1> {
      focus %W
      tabbedOptionFrame::select %W [%W nearest %y]
   }
   bind $plb <Down> {
      tabbedOptionFrame::select %W [%W index active]
   }
   bind $plb <Up> {
      tabbedOptionFrame::select %W [%W index active]
   }
   # Put this new binding at the end so the up/down fires after
   # the active line is active.
   set tags [bindtags $plb]
   set tags [concat [lrange $tags 1 end] [lindex $tags 0]]
   bindtags $plb $tags
   # Pack the selector list box
   grid $base.lb -row 0 -column 0 -sticky nsew
   # Expand data only on resize
   grid columnconfigure $base 1 -weight 1
   grid rowconfigure $base 0 -weight 1
   set TOF($base,lb) $plb
   # Insert the initial frame if it exists
   if {$i > 0} {
      $plb selection set 0
      $plb activate 0
      select $plb 0
   }
   return $base
}
#------------------------------------------------------------------------------
# Function   : tabbedOptionFrame::select
# Description: Callback when user selects a option
# Author     : Tom Wilkason
# Date       : 5/15/2002
#------------------------------------------------------------------------------
proc tabbedOptionFrame::select {lb index} {
   variable TOF
   if {[info exists TOF($lb,currentFrame)]} {
      grid forget $TOF($lb,currentFrame)
   }
   grid $TOF($lb,$index,frame) -row 0 -column 1 -sticky nsew
   set TOF($lb,currentFrame) $TOF($lb,$index,frame)
}
#------------------------------------------------------------------------------
# Function   : tabbedOptionFrame::getHull
# Description: Return the hull for some index in an option frame
# Author     : Tom Wilkason
# Date       : 5/15/2002
#------------------------------------------------------------------------------
proc tabbedOptionFrame::getHull {base index} {
   variable TOF
   set lb $TOF($base,lb)
   if {[info exists TOF($lb,$index,hull)]} {
      return $TOF($lb,$index,hull)
   } else {
      return -code error "$base with index $index does not exist"
   }
}
#------------------------------------------------------------------------------
# Function   : tabbedOptionFrame::getFrame
# Description: Return the frame for some index in an option frame
# Author     : Tom Wilkason
# Date       : 5/15/2002
#------------------------------------------------------------------------------
proc tabbedOptionFrame::getFrame {base index} {
   variable TOF
   set lb $TOF($base,lb)
   if {[info exists TOF($lb,$index,frame)]} {
      return $TOF($lb,$index,frame)
   } else {
      return -code error "$base with index $index does not exist"
   }
}
#------------------------------------------------------------------------------
# Function   : tabbedOptionFrame::lock
# Description: Lock the minimimum size of each tab to the size of the largest
#              tab.
# Author     : Tom Wilkason
# Date       : 5/15/2002
#------------------------------------------------------------------------------
proc tabbedOptionFrame::lock {base} {
   variable TOF
   set i 0
   update idletasks
   set maxHeight 0
   set maxWidth 0
   foreach {tofIndex} [array names TOF "$base*,frame"] {
      set tof $TOF($tofIndex)
      set height [winfo reqheight $tof]
      set width [winfo reqwidth $tof]
      if {$height > $maxHeight} {set maxHeight $height}
      if {$width > $maxWidth} {set maxWidth $width}
   }
   grid columnconfigure $base 1 -minsize $maxWidth
   grid rowconfigure $base 0 -minsize $maxHeight
}
#----------------------------------------------------------------------------
# Remove all memory
#----------------------------------------------------------------------------
proc tabbedOptionFrame::close {base} {
   variable TOF
   set i 0
   update idletasks
   set maxHeight 0
   set maxWidth 0
   array unset TOF $base*
}

#------------------------------------------------------------------------------
# Function   : waitWindow namespace
# Description: Functions to pop-up a window while stuff is going on
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
namespace eval waitWindow {
   variable W .waitWindow
   variable linkedText "Please Wait"
   variable width 60
   variable meter 0
   variable BREAK 0
   variable wprg
   variable startTime [clock seconds]
}

#------------------------------------------------------------------------------
#
#------------------------------------------------------------------------------
proc waitWindow::ifbreak {args} {
   update
   variable BREAK
   if {$BREAK} {
      set BREAK 0
      return 1
   } else {
      return 0
   }
}
proc waitWindow::init {args} {
   variable W
   variable width
   variable linkedText
   variable wprg
   variable BREAK
   variable stop
   variable perctext
   variable counterText ""
   set splash [toplevel $W -class SnackAmp]
   wm withdraw $W
   catch {wm attributes $W -toolwindow 1}
   wm title $W "SnackAmp $::SAVersion"
   wm protocol $W WM_DELETE_WINDOW [list wm withdraw $W]
   grid columnconfigure $W 0 -weight 1
   set i 0

   # Status
   set wtxt [label $W.lab -anchor w -justify left -pady 4 -padx 2 -textvariable waitWindow::linkedText -width [expr {$width+3}]]
   grid $wtxt -sticky ew

   # Outer frame here
   set wbody [labeledFrame $W.lf -text "Progress" -bd 2 -relief groove]
   grid $W.lf -sticky nsew


   # ProgressBar here
   set wprg [ProgressBar $wbody.pb -type infinite -maximum 100 -variable waitWindow::meter]
   # Percent Status
   set perctext [label $wbody.lab -anchor w -justify left -pady 4 -padx 2 -textvariable waitWindow::counterText]

   grid $wprg -sticky ew -padx 2 -pady 4
   grid $perctext -sticky w
   grid columnconfigure $wbody 0 -weight 1

   set stop [button $W.stop -text "Cancel" -relief raised -compound left -image saImages::Close \
      -pady 0 -padx 4 -command {set waitWindow::::BREAK 1}]
   grid $stop
   update idletasks
   # center it
   wm geometry $W +[expr {([winfo screenwidth .]-[winfo reqwidth $W])/2}]+[expr {([winfo screenheight .]-[winfo reqheight $W])/2}]
   # This don't seem to work, maybe because iconified?
   ontop::pinOnSystemMenu {} $W
   myIcon $W
}
proc waitWindow::grab {args} {
   variable W
   variable width
   catch {::grab set -global $W}
}

proc waitWindow::busyme {args} {
   variable meter
   after cancel waitWindow::busyme
   set meter 1
   after 500 waitWindow::busyme
}
proc waitWindow::release {args} {
   variable W
   variable width
   catch {::grab release $W}
}
#----------------------------------------------------------------------------
# Show some information in a pop-up windows
#----------------------------------------------------------------------------
proc waitWindow::bump {args} {
   variable meter
   set meter 1
   update
}
proc waitWindow::resize {args} {
   variable W
   wm geometry $W {}
}
proc waitWindow::show {Text {stoppable 0}} {
   variable W
   variable linkedText
   variable width
   variable wprg
   variable meter
   variable stop
   variable perctext
   variable counterText
   if {$stoppable} {
      grid $stop
   } else {
      grid forget $stop
   }
   grid forget $perctext
   $wprg configure -type infinite
   wm geometry $W {}
   if {[wm state $W] != "normal"} {
      wm deiconify $W
   } else {
      raise $W
   }
   set linkedText $Text
   set meter 1
   waitWindow::busyme
   update
}
#----------------------------------------------------------------------------
# Display a progress bar with % complete
#----------------------------------------------------------------------------
proc waitWindow::percent {value max Text {stoppable 0}} {
   variable W
   variable linkedText
   variable width
   variable wprg
   variable meter
   variable stop
   variable perctext
   variable counterText
   variable startTime
   after cancel waitWindow::busyme
   if {$max==0} {
      incr max
      debug "Invalid Max, can't equal 0"
   }
   if {$stoppable} {
      grid $stop
   } else {
      grid forget $stop
   }
   $wprg configure -type normal
   if {[wm state $W] != "normal"} {
      wm deiconify $W
   } else {
      raise $W
   }
   set linkedText $Text
   set counterText "Progress: $value of $max"
   grid $perctext  -sticky w
   set meter [expr {$value*100/$max}]
   update
}

proc waitWindow::close {args} {
   variable W
   variable BREAK 0
   after cancel waitWindow::busyme
   set counterText ""
   wm withdraw $W
   #update
}
#------------------------------------------------------------------------------
# Lock the minimum toplevel size to required size
#------------------------------------------------------------------------------
proc lockSize {B} {
   update idletasks
   wm minsize $B [winfo reqwidth $B] [winfo reqheight $B]
}
#------------------------------------------------------------------------------
# Set the focus to a widget, but ignore any errors
# Used to traverse up/down a column of entry boxes
#------------------------------------------------------------------------------
proc Focus {widget} {
   if {[winfo exists $widget]} {
      focus $widget
   } else {
      bell
   }
}

#------------------------------------------------------------------------------
# Function   : killAfter
# Description: Kill some lingering after loops
# Author     : Tom Wilkason
# Date       : 8/4/2001
#------------------------------------------------------------------------------
proc killAfter {name} {
   foreach one [after info] {
      if {[lindex [after info $one] 0] == $name} {
         after cancel $one
      }
   }
}
#----------------------------------------------------------------------------
# Cancel an after based on a pattern
#----------------------------------------------------------------------------
proc afterCancel {pattern} {
   foreach aft [after info] {
      set afinfo [lindex [after info $aft] 0]
      if {[string match $pattern $afinfo]} {
         after cancel $afinfo
         return 1
      }
   }
   return 0
}
#------------------------------------------------------------------------------
# Function   : amAfterExists
# Description: Determine if an after loop is running for some process
# Author     : Tom Wilkason
# Date       : 6/21/1999
#------------------------------------------------------------------------------
proc afterExists {Proc {arg ""}} {
   foreach aft [after info] {
      set afinfo [lindex [after info $aft] 0]
      if {[string length $arg] == 0} {
         if {[string equal [lindex $afinfo 0] $Proc]} {
            return 1
         }
      } else {
         if {[string equal [lindex $afinfo 0] $Proc] && [string equal [lindex $afinfo 1] $arg]} {
            return 1
         }
      }
   }
   return 0
}
#------------------------------------------------------------------------------
# Function   : centerWindow
# Description: Center a window on the screen
# Author     : Tom Wilkason
# Date       : 11/3/2001
#------------------------------------------------------------------------------
proc centerWindow {w} {
   wm withdraw $w
   update idletasks
   wm geometry $w "+[expr {([winfo screenwidth .]-[winfo reqwidth $w])/2}]+[expr {([winfo screenheight .]-[winfo reqheight $w])/2}]"
   wm deiconify $w
}
#------------------------------------------------------------------------------
# Function   : APLExists
# Description: Return a bool to denote the state of the Autoplaylist
# Author     : Tom Wilkason
# Date       : 6/20/2002
#------------------------------------------------------------------------------
proc APLExists {args} {
   return [expr {[info exists ::PLB] && [winfo exists $::PLB]}]
}
#------------------------------------------------------------------------------
# Function   : ontop namespace
# Description: Provide windows functions to pin a window on top
# Author     : Tom Wilkason
# Date       : 7/26/2002
#------------------------------------------------------------------------------
namespace eval ontop {
   variable enabled 0
   foreach {left top right bottom} {0 0 0 0} {
      set ::TA(screen_left) 0
      set ::TA(screen_top) 0
      set ::TA(screen_right) [winfo screenwidth .]
      set ::TA(screen_bottom) [expr {[winfo screenheight .]-50}]
   }
   if {$::tcl_platform(platform)=="windows"} {
      if {[catch {package require TkWinTools} result]} {
         puts $result
         proc control {win how} {
            saLog "tkwintools not found"
         }
         proc state {win} {
            saLog "tkwintools not found"
         }
         proc ifpossible {args} {
            return 0
         }
      } else {
         proc ifpossible {W top} {
            if {[catch {ontop::state $W} result] } then {
               return 0
            } else {
               ontop::control $top 1
               return 1
            }
         }
         proc control {win how} {
            if {[winfo exists $win] && [winfo ismapped $win]} {
               tkwintools::topmost [winfo toplevel $win] $how
            }
         }
         proc state {win} {
            if {[winfo exists $win] && [winfo ismapped $win]} {
               return [tkwintools::topmost [winfo toplevel $win]]
            }
         }
         foreach {left top right bottom} [tkwintools::sysinfo -workarea] {
            set ::TA(screen_left) $left
            set ::TA(screen_top) $top
            set ::TA(screen_right) $right
            set ::TA(screen_bottom) $bottom
         }
         set enabled 1
      }
   } else {
      proc ifpossible {args} {
         return 0
      }
   }
   # Put a pin on top menu on the system menu bar under windows
   proc pinOnSystemMenu {M W} {
      variable enabled
      if {$enabled} {
         update idletasks
         if {$M=={}} {
            set M $W.menubar
            menu $M -tearoff 0
            $W config -menu $M
         }
         ##
         # On top checkbutton
         #
         $M add cascade -label System -menu $M.system
         menu $M.system -tearoff 0
         set ::TA($W,pinned) 0
         $M.system add separator
         $M.system add checkbutton -label "Always on Top" \
            -variable ::TA($W,pinned) \
            -command "ontop::control $W \$::TA($W,pinned)"
         ##
         # Transparency
         #
         menu $M.trans
         fade::setDefTrans $W
         $M.system add cascade -label "Transparency" -menu $M.trans
         for {set i 0.3} {$i <= 1.01} {set i [expr {$i+0.05}]} {
            if {$i>0.951} {
               set i 1.0
               set lab "Normal"
            } else {
               set lab "[expr {$i*100}]%"
            }
            $M.trans add radiobutton -label $lab -value $i\
               -variable snackAmpSettings($W,transparency) \
               -command [list fade::setTrans $W $i]
         }
      }
   }
}
#------------------------------------------------------------------------------
# This fixes a bug in 8.4 that posted windows menus incorrectly
#------------------------------------------------------------------------------
if {$::tcl_version==8.4} {
   proc ::tk::PostOverPoint {menu x y {entry {}}}  {
      global tcl_platform

      if {[string compare $entry {}]} {
         if {$entry == [$menu index last]} {
            incr y [expr {-([$menu yposition $entry] \
               + [winfo reqheight $menu])/2}]
         } else {
            incr y [expr {-([$menu yposition $entry] \
               + [$menu yposition [expr {$entry+1}]])/2}]
         }
         incr x [expr {-[winfo reqwidth $menu]/2}]
      }
      $menu post $x $y
      if {$entry ne "" && [$menu entrycget $entry -state] ne "disabled"} {
         $menu activate $entry
         GenerateMenuSelect $menu
      }
   }
}
namespace eval levelmeter {
   variable left {}
   variable right {}
}
#------------------------------------------------------------------------------
# Left/Right channel meter
#
# -width 20 -length 200 -orient vertical -oncolor green
#------------------------------------------------------------------------------
proc levelmeter::create {widget args} {
   variable left $widget.left
   variable right $widget.right
   frame $widget
   grid [eval snack::levelMeter {$left} $args] \
        [eval snack::levelMeter {$right} $args] -sticky nsew
   return $widget
}
#------------------------------------------------------------------------------
# Compute the current sample levels
#------------------------------------------------------------------------------
proc levelmeter::sampleLevel {snd _left _right} {
   upvar $_left left
   upvar $_right right
   if {[snack::audio active]} {
      if {[catch {
         set level $::TA(line,$::snackAmpSettings(lineSelect))
         set here [$snd current_position] ;# this data is buffer time into the future
         set slevel [$snd sample $here]
         switch -- [llength $slevel] {
            1 {
                  set left [expr {abs($slevel*$level/100.)}]
                  set right [expr {abs($slevel*$level/100.)}]
            }
            2 {
               foreach {s1 s2} $slevel {
                  set left [expr {abs($s1*$level/100.)}]
                  set right [expr {abs($s2*$level/100.)}]
               }
            }
            default {}
         }
      } result]} {
         set left 0
         set right 0
      }
   } else {
      set left 0
      set right 0
   }
}
#------------------------------------------------------------------------------
# Update the meter with current sample levels
#------------------------------------------------------------------------------
proc levelmeter::cmUpdate {args} {
   variable left
   variable right
   if {[winfo exists $left] && [winfo ismapped $left]} {
      set snd [soundControl::sound]
      sampleLevel $snd l r
      $left  configure -level $l
      $right configure -level $r
      after $::snackAmpSettings(meterUpdateRate) levelmeter::cmUpdate
   } else {
      after 3000 levelmeter::cmUpdate
   }
}
#------------------------------------------------------------------------------
# Combo box to replace the option menu
#------------------------------------------------------------------------------
proc listOptionMenu {w varName choices {width 10}} {
   if {[llength $choices]>20} {
      set height 20
   } else {
      set height [llength $choices]
   }
   ComboBox $w \
           -borderwidth 1 \
           -width $width \
           -height $height \
           -hottrack true \
           -textvariable $varName \
           -editable false \
           -values $choices

  $w.e configure -background $::snackAmpSettings(ebg) -foreground $::snackAmpSettings(efg)
  settings::regColor $w.e -foreground ::snackAmpSettings(efg) \
   -background ::snackAmpSettings(ebg)
  return $w
}
#------------------------------------------------------------------------------
# Function   : Wait
# Description: Change the cursor to hourglass if needed
# Author     : Tom Wilkason
# Date       : 1/5/2001
#------------------------------------------------------------------------------
proc Wait {{how 0}} {
   global TA
   set cursor $TA(arrow)
   switch -- $how {
      1 {
         set cursor watch
      }
      0 {
         set cursor $TA(arrow)
      }
      default {}
   }
   foreach {child} [winfo children .] {
      $child config -cursor $cursor
   }
   if {[APLExists]} {
      $::TREE config -cursor $cursor
   }
   #update
}

#------------------------------------------------------------------------------
# Change the mouse cursor to show an action
#------------------------------------------------------------------------------
proc flashMouse {args} {
   if {![catch {winfo toplevel [focus]} tl] } then {
      $tl config -cursor exchange
      update
      after 100 [list $tl config -cursor $::TA(arrow)]
   }
}
#------------------------------------------------------------------------------
# trackTransition namespace
#------------------------------------------------------------------------------
namespace eval trackTransition {
   variable MaxX
   variable MaxY
   variable MinY
   variable MinX
   variable lastInID 0
   variable lastOutID 0
   variable tl .transition
   variable songArray
   upvar #0 ::songArray songArray
}

#------------------------------------------------------------------------------
#
#------------------------------------------------------------------------------
proc trackTransition::show {{delta 5}} {
   variable MaxX
   variable MaxY
   variable MinY
   variable MinX
   variable tl
   variable lastInID
   variable lastOutID
   global snackAmpSettings
   if {![winfo exists $tl]} {
      toplevel $tl -padx 10 -pady 10 -background $::snackAmpSettings(bg) -bd 3 -relief ridge
      settings::regColor $tl -background ::snackAmpSettings(bg)
      wm withdraw $tl
      wm overrideredirect $tl 1
      frame $tl.left -background $::snackAmpSettings(bg)
      frame $tl.right -background $::snackAmpSettings(bg)
      settings::regColor $tl.left ::snackAmpSettings(bg)
      settings::regColor $tl.right ::snackAmpSettings(bg)
      label $tl.left.logo -image saImages::saLogo  -background $::snackAmpSettings(bg)
      settings::regColor $tl.left.logo ::snackAmpSettings(bg)
      pack $tl.left.logo
      array set color {
         songLabel  tfg
         songArtist afg
         songInfo   fg
      }
      foreach {Title Field font} {
         Title  Title  songLabel
         Artist Artist songArtist
         Album  Album  songInfo
      } {

         label $tl.right.dt$Title -textvariable trackTransition::songArray($Field) \
            -font $font -pady 0  -bd 0 \
            -background $::snackAmpSettings(bg) \
            -foreground $::snackAmpSettings($color($font)) \
            -anchor w -bd 0
         settings::regColor $tl.right.dt$Title \
            -background ::snackAmpSettings(bg) \
            -foreground ::snackAmpSettings($color($font))
         pack $tl.right.dt$Title -fill both -expand true
      }
      # Get miniplayer size  and position at bottom right of usable area
      # Do the fading
      pack $tl.left -side left -expand false -padx 10
      pack $tl.right -side right -expand true
   }
   update
   # Don't place over taskbar
   # Locate the player in the right hand bottom corner then pop it up.
   set MaxX [expr {$::TA(screen_right)-[winfo reqwidth $tl]}]
   set MaxY [expr {$::TA(screen_bottom)-[winfo reqheight $tl]}]
   set MinX [winfo screenwidth .]
   set MinY [winfo screenheight .]
   #Move it down
   wm geometry $tl "+$MaxX+$MinY"
   wm deiconify $tl
   # Windows unique
   ontop::ifpossible $tl $tl
   # Make 0.8 ms a setting
   catch {wm attributes $tl -alpha .80}
   after cancel $lastInID
   after cancel $lastOutID
   set lastInID [slideIn $tl $delta $MaxY]
   # Make 6000 ms a setting
   set lastOutID [after 6000 [list trackTransition::slideOut $tl $delta $MinY]]
}
proc trackTransition::slideIn {W delta max} {
   variable lastInID
   after cancel $lastInID
   foreach {w h x y} [split [wm geometry $W] "x +"] {break}
   # if y below max
   if {$y > $max} {
      #push up
      incr y -$delta
      wm geometry $W "+$x+$y"
      set lastInID [after 20 [list trackTransition::slideIn $W $delta $max]]
      update idletasks
   }
   return $lastInID
}
proc trackTransition::slideOut {W delta max} {
   variable lastOutID
   variable lastInID
   after cancel $lastOutID
   after cancel $lastInID
   foreach {w h x y} [split [wm geometry $W] "x +"] {break}
   # if y above max
   if {$y < $max} {
      # push down
      incr y $delta
      wm geometry $W "+$x+$y"
      set lastOutID [after 20 [list trackTransition::slideOut $W $delta $max]]
      update idletasks
   }
   return $lastOutID
}
#----------------------------------------------------------------------------
# Simple note editor to change wrapped text in some widgets
#----------------------------------------------------------------------------
namespace eval noteEdit {
   variable tag "noteEdit"
}
#------------------------------------------------------------------------------
# Simple editor to edit a note field wrapped
#------------------------------------------------------------------------------
proc noteEdit::create {_var} {
   variable tag
   set i 0
   upvar $_var var
   while {[winfo exists .ne$i]} {incr i}
   set W .ne$i
   toplevel $W
   wm iconify $W
   wm title $W "Note Editor"
   wm protocol $W WM_DELETE_WINDOW [list noteEdit::close $W]
   text $W.en -width 70 -height 8 -wrap word
   frame $W.bt
   pack [button $W.bt.cancel -text "Cancel" -relief raised  -padx 4\
      -compound left -image saImages::Close -command [list noteEdit::close $W]] \
      -side left  -padx 4 -pady 2
   pack [button $W.bt.ok -text "OK" -relief raised -padx 4\
      -compound left -image saImages::Apply -command [list noteEdit::ok $W.en $_var]] \
      -side left  -padx 4 -pady 2
   pack $W.bt -side bottom
   pack $W.en -side top -expand true -fill both
   $W.en insert end $var
   $W.en configure -undo 1
   bind $W.en <Control-z> [list noteEdit::undo $W.en]
   settings::restoreSize $W $tag
   wm deiconify $W
}
proc noteEdit::ok {w _var} {
   upvar $_var var
   set var [string trim [$w get 1.0 end]]
   close $w
}
proc noteEdit::undo {w} {
   catch {$w edit undo}
}
proc noteEdit::close {w} {
   variable tag
   set W [winfo toplevel $w]
   settings::pushSize $W $tag
   destroy $W
}
#------------------------------------------------------------------------------
# An entry history widget, keeps track of what the user entered into it
# by linking a global variable to it.
#------------------------------------------------------------------------------
namespace eval entryHistory {
   variable maxEntries 20
}
proc entryHistory::create {w tv var} {
   variable maxEntries
   upvar #0 $tv      ::entryHistory::${w}.selected
   upvar #0 $var     ::entryHistory::${w}.choices
   set cb [ComboBox $w \
     -borderwidth 1 \
     -height $maxEntries \
     -width 9 \
     -autocomplete true \
     -command [list entryHistory::push $w] \
     -textvariable $tv \
     -hottrack true \
     -editable true \
     -values {} \
     -postcommand [list entryHistory::populate $w]]
   set hull [entryHistory::hull $w]
   settings::regColor $hull \
      -background ::snackAmpSettings(ebg) -foreground ::snackAmpSettings(efg)
      # Linux doesn't pick up option database correctly
   [entryHistory::hull $w] configure -background $::snackAmpSettings(ebg) \
      -foreground $::snackAmpSettings(efg)
   # Needed for unix
   bind $hull <1> [list focus $hull]
  return $cb
}
proc entryHistory::hull {w} {
   # return the entry field to bind to
   return $w.e
}
proc entryHistory::selected {w} {
   upvar ::entryHistory::${w}.selected selected
   return $selected
}
proc entryHistory::populate {w} {
   upvar ::entryHistory::${w}.choices choices
   $w configure -values $choices
}
proc entryHistory::push {w} {
   variable maxEntries
   upvar ::entryHistory::${w}.selected selected
   upvar ::entryHistory::${w}.choices choices
   if {$selected!=[lindex $choices 0]} {
      lpush left choices $selected
      set choices [lrange $choices 0 $maxEntries]
   }
}

#------------------------------------------------------------------------------
#
#------------------------------------------------------------------------------
proc hmtest {args} {
   set W .bubba
   destroy $W
   toplevel $W
   pack [entryHistory::create $W.eh ::TA(bubba) ::snackAmpSettings(bubbaHistory)] -expand true
   bind [entryHistory::hull $W.eh] <Return> [list entryHistory::push $W.eh]
}

#------------------------------------------------------------------------------
#
#------------------------------------------------------------------------------
proc hmprocess {w} {
   set selected [entryHistory::selected $w]
   puts "Processing $selected"
   entryHistory::push $w
}
#------------------------------------------------------------------------------
# Return a horizontal line (used for some dialogs and such)
#------------------------------------------------------------------------------
proc horzLine {win} {
   return [frame $win -height 2 -borderwidth 3 -relief sunken]
}

#------------------------------------------------------------------------------
#  Return the selected tracks from a listbox
#------------------------------------------------------------------------------
proc getSelectedTracks {lb} {
   if {[llength [$lb curselection]] > 0} {
      foreach {ind} [lsort -integer [$lb curselection]] {
         switch -glob -- $lb {
            ".saTop*"  {lappend data [LB::keyToName [LB::keyFromIndex $ind]]}
            ".wFC*"    {lappend data [aplLB::rowIndex2file $ind]}
         }
      }
      return $data
   }
}

#----------------------------------------------------------------------------
# Fade in/out routines, need transparency support
#----------------------------------------------------------------------------
namespace eval fade {
   variable fade
   variable fades
   variable fadeImages
   upvar #0 ::snackAmpSettings(fadeImages) fadeImages
   upvar #0 ::snackAmpSettings(fadeImages) fadeImages
   if {[catch {wm attributes . -alpha}] } then {
      proc in {window args} {if {[winfo exists $window]} {wm deiconify $window}}
      proc out {window args} {if {[winfo exists $window]} {wm withdraw $window}}
      proc off {window args} {if {[winfo exists $window]} {wm withdraw $window}}
      proc in_block {window args} {if {[winfo exists $window]} {wm deiconify $window}}
      proc out_block {window args} {if {[winfo exists $window]} {wm withdraw $window}}
      proc setDefTrans {args} {}
      proc setTrans {args} {}
      set fades 0
   } else {
      # wm attributes work
      proc setDefTrans {W} {
         upvar ::snackAmpSettings($W,transparency) mytrans
         if {! ([info exists mytrans] && [string is double -strict $mytrans])} {
            set mytrans  1.0
         } else {
            wm attributes $W -alpha $mytrans
         }
      }
      proc setTrans {W i} {
         wm attributes $W -alpha $i
      }
      set fades 1
      proc in {window {delay 10} {max 1.0}} {
         variable fadeImages
         if {$fadeImages} {
            if {[winfo exists $window]} {
               # limit fading in to maximum value for that window
               upvar ::snackAmpSettings($window,transparency) mytrans
               if {!([info exists mytrans] && [string is double -strict $mytrans])} {
                  set mytrans  $max
               } else {
                  set max $mytrans
               }
               set level [wm attributes $window -alpha]
               set newLevel [expr {$level + 0.09}]
               if {$newLevel < $max} {
                  wm attributes $window -alpha $newLevel
                  catch {after cancel $::fade::after($window)}
                  set ::fade::after($window) [after $delay [list fade::in $window $delay $max]]
               }
            }
         } else {
            if {[winfo exists $window]} {
               #wm attributes $window -alpha 1.0
               wm deiconify $window
            }
         }
      }
      proc out {window {delay 10}} {
         variable fadeImages
         if {$fadeImages} {
            if {[winfo exists $window]} {
               set level [wm attributes $window -alpha]
               set newLevel [expr {$level - 0.09}]
               if {$newLevel >= 0.0} {
                  wm attributes $window -alpha $newLevel
                  catch {after cancel $::fade::after($window)}
                  set ::fade::after($window) [after $delay [list fade::out $window $delay]]
               } else {
                  wm withdraw $window
               }
            }
         } else {
            if {[winfo exists $window]} {
               wm withdraw $window
               #wm attributes $window -alpha 1.0
            }
         }
      }
      proc in_block {window {delay 10}} {
         variable fadeImages
         # Check if option enabled
         # 8.4.9 windows Bug: when transitioning from .999 to 1.0 a slight flicker appears
         # also, the menus aren't always restored when not transitioning to 1.0
         if {$fadeImages && [winfo exists $window]} {
            for {set i [wm attributes $window -alpha]} {$i < 1.0} {set i [expr {$i+0.09}]} {
               wm attributes $window -alpha $i
               after $delay
            }
            wm attributes $window -alpha .99999
         }
      }

      proc out_block {window {delay 10}} {
         variable fadeImages
         # Check if option enabled
         if {$fadeImages} {
            if {[winfo exists $window]} {
               for {set i [wm attributes $window -alpha]} {$i > 0.0} {set i [expr {$i-0.09}]} {
                  wm attributes $window -alpha $i
                  after $delay
               }
               wm withdraw $window
            }
         } else {
            wm withdraw $window
            #wm attributes $window -alpha 1.0
         }
      }
      proc off {window} {
         variable fadeImages
         if {$fadeImages} {
            wm attributes $window -alpha 0.0
         }  else {
            # Don't do anything
         }
      }
   }
}

package provide gpWidgets 1.0
