#------------------------------------------------------------------------------
# snackAmp - saGui package 1.0 for snackAmp
# provides main user interface GUI
#
# Music File Player in Tcl/Tk
#
# Copyright (C) 2001 Tom Wilkason
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# Please send any comments/bug reports to
# tom.wilkason@cox.net  (Tom Wilkason)
#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
# Define the procedures and continue processing at the end of the file
#------------------------------------------------------------------------------
namespace eval gui {
   variable tsScale
   variable volScale
   variable trackPosition 0
   variable lastFolderName ""
   variable songDuration 0  ;# this might be better in soundControl
   if {[catch {package require Img} result] } then {
      variable hasImg 0
   } else {
      variable hasImg 1
   }
   variable saLogo {
      R0lGODlhIAAgAKIAAACAgAAA////a7HH/wAxlgAAgGuP/0hz/yH5BAEAAAAA
      LAAAAAAgACAAAAPDCLrc/k2INuqAmMmtRghVJgKb5H1gOGIm6lrX6pxubcnL
      SdS86kiUD2G44wkhpQlAtyDyCMpHktkgFAooKAcp0EWDAQKphPFuc8cFEMJM
      Aqm/7zKtmNIj5HkYaeZu4DNCSVxebGmDHTdma2h7Gko0KUduE4AMJwcHH4B2
      joEGRmJSfUGgT2WCeQpFpzmKh6pWNQWiepuWjACyH1cMHwamlg9EQw0BpjW1
      OL7ILsrLClhPz9AA0s7VDlfSV9TZ1tvf4gAJADs=
   }

}
#------------------------------------------------------------------------------
# Function   : buildGui
# Description: Build the player Gui
# Author     : Tom Wilkason
# Date       : 12/27/2000
#------------------------------------------------------------------------------
proc gui::buildGui {args} {
   variable tsScale
   variable volScale
   variable trackPosition
   variable hasImg
   global TA LB
   set ::songArray(Title) "SnackAmp Music Player"
   set TA(songTitle) "SnackAmp Music Player"
   set TA(playlistEntry) "SnackAmp Music Player"
   set TA(songBalloon) "Track Details"

   ##
   # Hide the toplevel and move it off the visible screen
   #
   toplevel .saTop -class SnackAmp -cursor $::TA(arrow) -borderwidth 0
   set ::SA .saTop
   wm title $::SA "SnackAmp Music File Player"
   ##
   # clean up nicely
   #
   wm protocol $::SA WM_DELETE_WINDOW {taExit}
   wm protocol . WM_DELETE_WINDOW {taExit}
   wm withdraw $::SA
   # Init the volumn linkages

   gui::initVolume
   ##
   # Control Panel
   #
   pack [frame $::SA.panel -borderwidth 2 -relief groove] -side top -fill x -expand false -anchor w
   frame $::SA.panel.stat -relief sunken -borderwidth 2

   ##
   # Main area for music details
   #
   frame $::SA.panel.stat.l -background $::snackAmpSettings(bg) \
      -borderwidth 0 -width 70
   settings::regColor $::SA.panel.stat.l -background ::snackAmpSettings(bg)
   layoutPlayInfo $::snackAmpSettings(showDetail) ::songArray $::SA.panel.stat.l
   # Trace the showDetail settings to update the display detail immediatly from the settings menu
   trace variable ::snackAmpSettings(showDetail) w [list gui::reLayoutPlayInfo $::SA.panel.stat.l ::songArray]
   trace variable ::snackAmpSettings(showLabels) w [list gui::reLayoutPlayInfo $::SA.panel.stat.l ::songArray]

   ##
   # Time counter
   #
   label $::SA.panel.stat.t -background $::snackAmpSettings(bg) -foreground $::snackAmpSettings(sfg) -borderwidth 0 \
   -font timeStatus -height 1 -textvar TA(timeStatus) -anchor e
   settings::regColor $::SA.panel.stat.t -background ::snackAmpSettings(bg) -foreground ::snackAmpSettings(sfg)
   balloon::define $::SA.panel.stat.t "Time Playing\nTime Remaining\nTotal Time"

   ##
   # Time scale
   #
   set tsScale [scale $::SA.panel.stat.ts -borderwidth 0 -show no -orient horiz \
      -width 5 -len 200 -from 0. -to 60. \
      -var gui::trackPosition -digits 1 -sliderlength 25 -resolution 0.1 \
      -highlightcolor $::snackAmpSettings(bg) \
      -troughcolor $::snackAmpSettings(bg) -command gui::updateTime]

   settings::regColor $tsScale -troughcolor ::snackAmpSettings(bg) \
      -highlightcolor ::snackAmpSettings(bg)

   balloon::define $tsScale "Show/Adjust playback location within file"


   ##
   # Click in title edits ID3 tag
   #
   set TA(muteFactor) 1.0
   bind $::SA.panel.stat.l.dtTitle <ButtonPress-3> {EditPlaying}
   balloon::widgetDefine $::SA.panel.stat.l.dtTitle

   ##
   # Level Meter, on the left
   #
   frame $::SA.panel.stat.meter -background $::snackAmpSettings(bg) \
      -borderwidth 0
   settings::regColor $::SA.panel.stat.meter -background ::snackAmpSettings(bg)
   if {$::snackAmpSettings(levelMeter)} {
      after idle gui::showLevelMeter
   }

   # Place the labels next to each other
   grid $::SA.panel.stat.meter -column 0 -row 0 -rowspan 1 -sticky  news    ;# Meter
   grid $::SA.panel.stat.l -column 1 -row 0 -sticky nsew                ;# Song Title
   grid $::SA.panel.stat.t -column 2 -row 0 -rowspan 1 -sticky nsew     ;# Time Status
   grid $::SA.panel.stat.ts -columnspan 6 -sticky nsew     ;# Time Scale

   grid $::SA.panel.stat -row 0 -column 1 -columnspan 2 -sticky nsew;# Inner Grid
   # Player controls
   #
   snack::createIcons
   set tf $::SA.panel.player
   grid [frame $tf] -row 1 -column 1 -sticky news

   ##
   # Button data
   #
   set buttonData {
      bitmap snackPlayPrev       Prev                           "Play previous in list"
      bitmap snackPlay           {set ::TA(STOP) 0; PlayResume} "Play Selection"
      bitmap snackPause          Pause                          "Pause/Resume playback"
      bitmap snackStop           {set ::TA(STOP) 1;Stop}        "Stop playing"
      bitmap snackPlayNext       {set ::TA(STOP) 0;Next}        "Play next in list"
      image  saImages::openFile  OpenFile                       "Add a file to play list queue"
      image  saImages::coverart gui::ToggleArt                  "Toggle the display of album art"
      image  saImages::treeImage AutoPlaylist                   "Manage your music collection and define AutoPlaylists"
   }
   #   image  saImages::edit-16  {package require saSettings;settings::tabbedSettingsGui}  "Manage Tools"
   # build each one
   foreach {bmType File Command Help} $buttonData {
      set iroot [namespace tail $File]
      button  $tf.$iroot -$bmType $File -borderwidth 1 -command $Command
      #bind $tf.$iroot <ButtonPress-1> $Command
      pack $tf.$iroot -side left -padx 0 -pady 0 -fill y
      balloon::button $tf.$iroot $Help
   }
   # Put this button before tree button
   pack [menubutton $tf.sortMenu -relief flat -image saImages::shuffleImage -menu $tf.sortMenu.drop] \
      -before $tf.coverart -side left -fill both
   balloon::button $tf.sortMenu  "Select sort mode"
   set mb $tf.sortMenu.drop
   ;# Rating
   set pm $tf.rating.popup
   pack [menubutton $tf.rating -relief flat -image saImages::iconthumbsup -menu $pm] \
   -before $tf.coverart -side left -fill both
   set m [menu $pm \
   -postcommand [list gui::ratingPopup $pm]]
   balloon::button $tf.rating "Adjust rating of the playing track, the more stars the better."
   # History menu, one for button and one for RMB
   ##
   # Sort menus
   #
   package require subSorting
   menu $mb -postcommand [list subSort::getAllModes $mb]
   set TA(treeButton) $tf.treeImage
   set TA(artButton) $tf.coverart

   set TA(sortButton) $tf.sortImage
   set TA(shuffleButton) $tf.shuffleImage
   set TA(pauseButton) $tf.snackPause
   if {!$hasImg} {
      destroy $TA(artButton)
   }
   ##
   # Allow time to be adjusted for all the buttons
   #
   set TA(setdrag) 1
   bind $tsScale <ButtonPress-1> {set TA(setdrag) 0}
   bind $tsScale <ButtonRelease-1> {set TA(setdrag) 1; PlayOffset}
   bind $tsScale <ButtonPress-2> {set TA(setdrag) 0}
   bind $tsScale <ButtonRelease-2> {set TA(setdrag) 1; PlayOffset}
   bind $tsScale <ButtonPress-3> {set TA(setdrag) 0}
   bind $tsScale <ButtonRelease-3> {set TA(setdrag) 1; PlayOffset}
   ##
   # Scale show/adjust Sound level
   #
   set TA(gain) [macros::getPlayGain]
   if {$::TA(showTicks)} {
      set ti 20
      set wid 5
   } else {
      set ti 0
      set wid 10
   }
   set volScale [scale $::SA.panel.sv -tickinterval $ti -digits 0 \
         -show no -length 1i -orient horiz -width $wid -command {PlayGain} \
         -length 200 -var TA(gain) -from 0. -to 64. -sliderlength 25 \
         -highlightcolor $::snackAmpSettings(bg) \
         -troughcolor    $::snackAmpSettings(bg)]  ;#-cursor sb_h_double_arrow
   settings::regColor $volScale -troughcolor ::snackAmpSettings(bg) \
      -highlightcolor $::snackAmpSettings(bg)
   grid $volScale -column 2 -row 1 -sticky ew

   #grid $volScale -column 0 -row 0 -rowspan 3 -sticky ns
   balloon::define $volScale "Adjust the playback level"
   ##
   # Mousewheel adjust volume from toplevel
   #
   bind $tsScale <Any-Leave> {
      focus [winfo toplevel %W]
   }
   bind $tsScale <Any-Enter> {
      focus %W
   }
   bind $volScale <Any-Enter> {
      set TA(pushFocus) [focus]
      focus %W
   }
   # Reconnect the volume
   bind $volScale <ButtonPress-1> {
      gui::initVolume
   }
   bind $volScale <Any-Leave> {
      if {[string equal %W $TA(pushFocus)]} {
         focus [winfo toplevel %W]
      } else {
         if {[catch {focus $TA(pushFocus)}]} {
            focus [winfo toplevel %W]
         }
      }
   }
   ##
   # Bind the mouse roller wheel on the scales
   #
   if {$::tcl_platform(platform) != "windows"} {
      bind $volScale <Button-5> {bumpLevel -2}
      bind $volScale <Button-4> {bumpLevel 2}
      bind $tsScale <Button-5> {macros::skipPlay -5}
      bind $tsScale <Button-4> {macros::skipPlay 5}
   } else {
      bind $tsScale <MouseWheel> {
         set amount [expr {%D/abs(%D)*5}] ;# get +/- 2
         macros::skipPlay $amount
         break
      }
      bind $volScale <MouseWheel> {
         set amount [expr {%D/abs(%D)*1}] ;# get +/- 2
         macros::bumpLevel $amount
         break
      }
   }

   # Adjust grid resize behavior and make things span properly
   grid columnconfigure $::SA.panel.stat 0 -weight 0
   grid columnconfigure $::SA.panel.stat 1 -weight 1            ;# For the label to stretch
   grid columnconfigure $::SA.panel 0 -weight 0
   grid columnconfigure $::SA.panel 1 -weight 0
   grid columnconfigure $::SA.panel 2 -weight 1

   # Collapsable frame for playlist
   set CF [cframe $::SA.hull "Tracks"]
   pack $::SA.hull -side top -fill both -expand true
   if {![catch {package require tkdnd} result] } then {
      # Playlist Tracks
      dnd bindsource $::SA.hull.label text/uri-list {LB::dndSrcLBDrag %W}
      bind $::SA.hull.label <Alt-1> {dnd drag %W -actions copy;break}
      balloon::define $::SA.hull.label "Alt-Drag from this label to copy selected playlist files"
      # Current Track
      dnd bindsource $::SA.panel.stat.l.dtTitle text/uri-list {LB::currentTrack}
      bind $::SA.panel.stat.l.dtTitle <1> {dnd drag %W -actions copy;break}
   }
   set ::TA(CF) $CF
   cframe.close $CF

   ##
   # Filter box
   pack [frame $CF.filter -borderwidth 2 -relief groove] -side top -expand no -fill x

   pack [button $CF.filter.label -text "Filter Playlist" \
   -compound left -image saImages::filter \
   -pady 0 -padx 2 -font entry -command {
      filtering::pushFilter
      # Revert to apl if nothing to filter
      if {[string length $::TA(filter)]==0} {
         macros::applyCurrentAPL
      } else {
         macros::filterMusic replace playlist
      }
   } -borderwidth 1] \
      -side left -expand no -fill both
   balloon::button $CF.filter.label "Restrict playlist to those files matching pattern, if no filter text reapply current AutoPlaylist"

   ##
   # Filter entry
   pack [entryHistory::create $CF.filter.entry ::TA(filter) ::snackAmpSettings(History)] \
      -side left -expand yes -fill x
   set ::FI [entryHistory::hull $CF.filter.entry]
   $::FI config -background $::snackAmpSettings(ebg) -foreground $::snackAmpSettings(efg)
   balloon::define $::FI "Enter filter or search text and use buttons/menus to left or right to process. Double click to clear. You can use 'or' for boolean matches. Use * or ? for wildcards."

   ##
   # Dropdown button
   pack [menubutton $CF.filter.history -image saImages::dropdownImage -menu $CF.filter.history.list -relief flat] -side left
   balloon::button $CF.filter.history  "Options to retrieve prior filter entries or apply current entries against the current AutoPlaylist files (APL) or against all of the known files"
   # History menu, one for button and one for RMB
   menu $CF.filter.history.list -postcommand [list filtering::filterHistory $CF.filter.history.list]
   menu $::FI.popup -postcommand [list filtering::filterHistory $CF.filter.entry.popup]
   bind $::FI <ButtonPress-3> {
      tk_popup %W.popup %X %Y
   }

   ##
   # Filter against all or just auto playlist
    set TA(filterAllButton) $CF.filter.doall

#    pack [button $CF.filter.doall -image saImages::filterOff\
#       -command filtering::toggleFilter -borderwidth 1] \
#       -side left -expand no -fill x
#    balloon::button $CF.filter.doall "Push this to have filter applied to all files (by default), not just ones in the current AutoPlaylist"
   ##
   # LB::AppendFile/Replace buttons for filter
   pack [button $CF.filter.btApp -image saImages::appendImage -command {
      filtering::pushFilter
      macros::filterMusic append all
   } -borderwidth 1] -side left
   balloon::button $CF.filter.btApp "Append files matching filter to current playlist"

   pack [button $CF.filter.btQueue -image saImages::queue -command {
      filtering::pushFilter
      macros::filterMusic queue all
   } -borderwidth 1] -side left
   balloon::button $CF.filter.btQueue "Queue files matching filter to current playlist"

   pack [button $CF.filter.btRep -image saImages::replaceImage -command {
      filtering::pushFilter
      macros::filterMusic replace all
   } -borderwidth 1] -side left
   balloon::button $CF.filter.btRep "Replace playlist with files matching filter"
   ##
   # Find button
   #
   pack [button $CF.filter.btFind -image saImages::findImage -command {
      findNextSong
   } -borderwidth 1] -side left
   balloon::button $CF.filter.btFind "Find the next track in the playlist that matches the Filter"

   ##
   # Search Media Library
   #
   pack [button $CF.filter.doall -image saImages::filterOff\
      -command filtering::searchFilter -borderwidth 1] \
      -side left -expand no -fill x
   balloon::button $CF.filter.doall "Search Media Library fields for text to the left"
   ##
   # Scrolled Listbox
   #
   set LB [LB::create $CF.frame]
   ##
   # Default then User Key Assigments and menus
   #
   bindings::defaultHotKeys
   bindings::assignHotKeys
   gui::InstallMenu
   gui::InstallLBMenu $LB
   # The songlist packs last, so it is the one that contracts the most
   pack $CF.frame -expand yes -fill both -side top
   gui::fixMenuDetail $::snackAmpSettings(showDetail)

   ##
   # Callbacks to immediatly set the fonts when the user changes them
   #
   settings::installFontCallbacks
   ##
   # Create an icon if needed
   #
   if {$::tcl_platform(platform) == "windows"} {
      if {[catch {package require IconSupport} result]} {
         puts stderr $result
      } else {
         iconSupport
         after idle [list myIcon $::SA]
      }
   } else {
      after idle [list myIcon $::SA]
   }
   gui::hookFontSizeAdjust
   if {$hasImg && $::snackAmpSettings(showCoverArt)} {
      after 1000 gui::ToggleArt
   }
}
#------------------------------------------------------------------------------
# Toggle the display of album art
#------------------------------------------------------------------------------
proc gui::ToggleArt {args} {
   set B .albumart
   global TA
   if {![winfo exists $B]} {
      set ::snackAmpSettings(showCoverArt) 1
      gui::coverArtViewer
      gui::packCoverArt [soundControl::fileName]
      fade::off $B
      wm deiconify $B
      fade::in $B
      $TA(artButton) configure -relief sunken
   } else {
      if {[wm state $B]=="withdrawn"} {
         set ::snackAmpSettings(showCoverArt) 1
         gui::packCoverArt [soundControl::fileName]
         fade::off $B
         wm deiconify $B
         fade::in $B
         $TA(artButton) configure -relief sunken
      } else {
         fade::out_block $B
         set ::snackAmpSettings(showCoverArt) 0
         $TA(artButton) configure -relief flat
      }
   }
}
#------------------------------------------------------------------------------
# Create a view window for coverart
#------------------------------------------------------------------------------
proc gui::coverArtViewer {args} {
   global snackAmpSettings
   variable hasImg
   variable albumTracks
   variable lb
   variable saLogo
   set B .albumart
   if {$hasImg && ![winfo exists $B]} {
      toplevel $B  -class SnackAmp
      wm focusmodel $B active  ;# Allow the mouse roll bar to work
      wm withdraw $B
      wm title $B "Now Playing"
      wm protocol $B WM_DELETE_WINDOW gui::ToggleArt
      image create photo coverArtPhoto -data $saLogo
      button $B.drop -text "Query Art" -relief flat -padx 0  -compound left -image saImages::web \
         -command {gui::searchCoverArt}
      balloon::button $B.drop "Press to bring up a dialog to query google for album art"
      label $B.ca -image coverArtPhoto -highlightthickness 0 -border 2 -relief sunken -bg black
      balloon::define $B.ca "Double click to query cover art for this album.  Drag desired image from web browser to here to save it."


      set tf [frame $B.pc -relief flat]
      set buttonData {
         image saImages::mini_prev   Prev       "Play previous in list"
         image saImages::mini_play   PlayResume "Play Selection"
         image saImages::mini_pause  Pause      "Pause/Resume playback"
         image saImages::mini_stop   Stop       "Stop playing"
         image saImages::mini_next   Next       "Play next in list"
      }
      # build each button
      foreach {bmType File Command Help} $buttonData {
         set iroot [namespace tail $File]
         button  $tf.$iroot -$bmType $File -borderwidth 1 -command $Command
         #bind $tf.$iroot <ButtonPress-1> $Command
         pack $tf.$iroot -side left -padx 0 -pady 0 -fill y
         balloon::define $tf.$iroot $Help
      }
      # Volume Slider
      scale $tf.volume -show no -orient horizontal -width 8 -command {PlayGain}\
            -len 120 -var TA(gain) -from 0. -to 64. -sliderlength 25\
            -activebackground blue -troughcolor $::snackAmpSettings(bg)

      settings::regColor $tf.volume -troughcolor ::snackAmpSettings(bg) -highlightcolor ::snackAmpSettings(bg)
      balloon::define $tf.volume "Adjust the volume"
      pack $tf.volume -fill x -side left -expand true
      # Reconnect the volume
      bind $tf.volume <ButtonPress-1> {
         gui::initVolume
      }
      # Mousewheel support
      bind $tf.volume <Any-Enter> {
         focus %W
      }
      if {$::tcl_platform(platform) != "windows"} {
         bind $tf.volume <Button-5> [list bumpLevel -1]
         bind $tf.volume <Button-4> [list bumpLevel 1]
      } else {
         bind $tf.volume <MouseWheel> {
            set amount [expr {%D/abs(%D)*2}] ;# get +/- 2
            bumpLevel $amount
         }
      }

      # Track Listing
      set hull [cframe $B.lf "Album Tracks" -relief sunken -bd 2]

      listbox $hull.list -relief sunken -bd 2 -listvariable gui::albumTracks \
         -exportselection  no                \
         -activestyle      none            \
         -foreground $::snackAmpSettings(fg) \
         -background $::snackAmpSettings(bg) \
         -selectforeground $::snackAmpSettings(bg) \
         -selectbackground $::snackAmpSettings(fg)
      # Listbox popup menu
      settings::regColor $hull.list  \
         -foreground       ::snackAmpSettings(fg) \
         -background       ::snackAmpSettings(bg) \
         -selectforeground ::snackAmpSettings(bg) \
         -selectbackground ::snackAmpSettings(fg)
      set lb $hull.list
      pack $lb -expand true -fill both
      pack $B.drop $B.ca $B.pc $B.lf -expand true -fill both
      #pack $B.drop $B.ca $B.lf -expand true -fill both
      if {[info exists snackAmpSettings(albumartPos)] && $snackAmpSettings(restorePlayerPos)} {
         wm geometry $B $snackAmpSettings(albumartPos)
      }
      bind $B.ca <Double-ButtonPress-1> {
         gui::searchCoverArt
      }
      if {![catch {package require tkdnd} result] } then {
         update
         dnd bindtarget $B.ca  "UniformResourceLocator" <Drop> {gui::dndDropCoverArt %D}
         dnd bindtarget $B.ca  "text/uri-list"          <Drop> {gui::dndDropCoverArt %D}
         # Listbox
         dnd bindtarget $lb    "UniformResourceLocator" <Drop> {gui::dndDropCoverArt %D}
         dnd bindtarget $lb    "text/uri-list"          <Drop> {gui::dndDropCoverArt %D}
      }
      #wm deiconify $B
      bind $B <Configure> [list wm geometry $B {}]
      ontop::pinOnSystemMenu {} $B
      after 100 [list myIcon $B]
   }
}
#----------------------------------------------------------------------------
# Search for cover art for the currently playing track
#----------------------------------------------------------------------------
proc gui::searchCoverArt {args} {
   variable coverArtSearch
   set i 0
   if {[db::lookup [soundControl::fileName] data 1]>0} {
      browser::coverart data
   }
}

#------------------------------------------------------------------------------
#  Handler for dropping cover art on the album viewer
#------------------------------------------------------------------------------
proc gui::dndDropCoverArt {files} {
   Trace
   variable lastFolderName
   global TA
   set flag 0
   if {[llength $files]} {
      # Hack, if first file is a graphic, just replace the
      # folder image
      set ext [string tolower [file extension [lindex $files 0]]]
      switch -- $ext {
         .jpg -
         .png {
            set lastFolderName xxx
            set firstFile [lindex $files 0]
            if {[macros::copyImageToFolder [lindex $files 0] [file dirname $TA(currentMF)]]} {
               gui::packCoverArt $TA(currentMF)
            }
         }
      }
   }
   return 1
}
#------------------------------------------------------------------------------
# Insert the coverart for the currently playing track
# state is normal, iconic, or withdrawn
#------------------------------------------------------------------------------
proc gui::packCoverArt {track {fade 0}} {
   Trace
   variable hasImg
   variable lastFolderName
   variable albumTracks
   variable lb
   variable saLogo
   set clamp 24
   set B .albumart
   if {[file exists $track] && [winfo exists $B] && $hasImg && $::snackAmpSettings(showCoverArt)} {
      set folder [file dirname $track]
      if {[db::lookup $track Data 1]==0} {
         return
      }
      foreach {ca valid} [coverart::covername $folder] {break}
      Update
      if {[llength $ca]} {
         if {$lastFolderName!=$folder} {
            if {$fade} {
               fade::out_block $B
            }
            set currentImage [$B.ca cget -image]
            catch {image delete $currentImage}
            if {[catch {image create photo -file $ca} img] } then {
               saLog $img
               pack $B.drop -before $B.ca
               set img [image create photo -data $saLogo]
            } else {
               set img [gui::clampImageSize $img \
                  $::snackAmpSettings(minArtSize) $::snackAmpSettings(maxArtSize)]
               pack forget $B.drop
            }
         } else {
            # If processing the same folder, then use existing cover image
            set img [$B.ca cget -image]
         }
      } else {
         set img [image create photo -data $saLogo]
      }
      if {$valid==0} {
         pack $B.drop -before $B.ca -fill x -expand true
      }
      set albumTracks [list]
      wm title $B "$Data(Album) now playing"
      $B.ca configure -image $img
      set match [join [list $Data(Track) $Data(Title)] { - }]
      set i 0
      set ind -1
      Update
      foreach {pair} [lsort -dictionary [lrange [db::albumTracks $Data(Artist) $Data(Album) [file dirname $track]] 0 40]] {
         set trk [join $pair { - }]
         if {[string equal $match $trk]} {
            set ind $i
         }
         incr i
         lappend albumTracks $trk
      }
      wm geometry $B {}
# Hardcoded reference to listbox, should later be a real playlist with methods
      set wid [$lb cget -width]
      if {$wid < $clamp} {
         $lb configure -width $clamp
      }
      $lb configure -height [llength $albumTracks]
      if {$ind >= 0} {
         $lb selection set $ind
         $lb see $ind
      }
      set lastFolderName $folder
      if {$fade} {
         wm deiconify $B
         fade::in $B
      }
   }
}

#------------------------------------------------------------------------------
# Clamp an image size to some size, only works on whole integer ratios
# todo: Use combination of zoom/subsample to scale image quickly to
# the correct size. e.g. zoom 3x subsample 2x to get 1.5 increase
# need to find the greatest common divisor
#------------------------------------------------------------------------------
proc gui::clampImageSize {img min max} {
   set height [image height $img]
   set width [image width $img]
   if {$height < $min || $height > $max} {
      set nom  $max
      if {$nom < $height} {
         foreach {zoom shrink} [dbl2frac [expr {double($nom)/double($height)}] 0.2] {break}
      } else {
         foreach {zoom shrink} [dbl2frac [expr {double($nom)/double($height)}] 0.4] {break}
      }
      if {$zoom==0 || $shrink==0} {
         saLog "illegal scales zoom=$zoom shrink=$shrink height=$height nom=$nom"
         return
      }
      set toImage [image create photo]
      $toImage copy $img -zoom $zoom $zoom -subsample $shrink
#saLog "$toImage copy $img -zoom $zoom $zoom -subsample $shrink"
      image delete $img
      return $toImage
   }
   return $img
}
##
#RS has this tiny "rationalizer":
#
proc dbl2frac {dbl {eps 0.001}} {
  for {set den 1} {$den<1024} {incr den} {
     set num [expr {round($dbl*$den)}]
     if {abs(double($num)/$den - $dbl) < $eps} break
  }
  list $num $den
}


#------------------------------------------------------------------------------
# Initialize the mixer controls
#------------------------------------------------------------------------------
proc gui::initVolume {args} {
   foreach {line} [snack::mixer lines] {
      set ::TA(line,$line) [snack::mixer volume $line]
      if {[snack::mixer channels $line] == "Mono"} {
         snack::mixer volume $line ::TA(line,$line)
      } else {
         snack::mixer volume $line ::TA(line,$line) ::TA(line,$line)
      }
   }
}
#------------------------------------------------------------------------------
# Function   : setTime
# Description: Set the scale timer to some value
# Author     : Tom Wilkason
# Date       : 8/8/2003
#------------------------------------------------------------------------------
proc gui::setTime {val} {
   variable tsScale
   $tsScale set $val
}
#------------------------------------------------------------------------------
# Function   : InstallMenu
# Description: Build the menu bar at the top of the GUI
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc gui::InstallMenu {} {
   global TA LB snackAmpSettings tcl_platform
   bindings::init $::SA menuBindings
   set M $::SA.bRmenubar
   # Make menus as small as possible
   menu $M \
      -activebackground [$::SA cget -background] \
      -activeforeground [$::SA cget -highlightcolor] \
      -activeborderwidth 1 -font menu
   # attach it to the main window
   if {$::tcl_platform(platform)=="unix"} {
      $M configure -borderwidth 0   ;# for linux
   }
   $::SA config -menu $M
   # Create more cascade menus
   foreach {m u} {File 0 Edit 0 Playlists 0 AutoPlaylists 0 Tools 0 Help 0} {
      switch -- $m {
         Playlists {
            set $m [menu $M.m$m -postcommand [list gui::playListFiles $M.m$m]]
            set TA(plPopUp) $M.m$m
            bindings::addCommand $M.m$m "Clear" {Clear}
            bindings::addCommand $M.m$m "Open" {macros::ReadPlaylistFile {} replace} 0
            bindings::addCommand $M.m$m "Save all" {macros::SavePlaylistFile $::LB replace all $::TA(PlayListFile)} 1
            bindings::addCommand $M.m$m "Save all to..." {macros::SavePlaylistFile $::LB replace all}
            bindings::addCommand $M.m$m "Save selected to..." {macros::SavePlaylistFile $::LB replace selected}
            bindings::addCommand $M.m$m "Append all to..." {macros::SavePlaylistFile $::LB append all}
            $M.m$m add separator
            ##
            # Media Players
            #
            set pm [menu $M.m$m.medplay \
               -postcommand [list gui::mediaPlayerPopup $M.m$m.medplay {selectedTracks $::LB}]]
            $M.m$m add cascade -label "Copy selected to Media Player" -menu $pm \
               -underline 0 -compound left -image saImages::copy
            $M.m$m add cascade -label "Save selected to..." -menu [menu $M.m$m.uReplace] -compound left -image saImages::SaveAs
            $M.m$m add cascade -label "Append selected to..." -menu [menu $M.m$m.uAppend -tearoff 1] -compound left -image saImages::SaveAs
            $M.m$m add separator
            $M.m$m add cascade -label "Append from..." -underline 0 -menu [menu $M.m$m.mAppend] -compound left -image saImages::appendImage
            $M.m$m add cascade -label "Insert from..." -underline 0 -menu [menu $M.m$m.mInsert] -compound left -image saImages::queue
            $M.m$m add cascade -label "Replace from..." -underline 0 -menu [menu $M.m$m.mReplace] -compound left -image saImages::replaceImage
            $M.m$m add separator
            set plrem [menu $M.m$m.remove -postcommand "[list gui::PLFilesRemove $M.m$m.remove]" ]
            $M.m$m add cascade -label "Remove..." -menu $plrem  -compound left -image saImages::delete
         }
         AutoPlaylists {
            set TA(FolderMenu) $m
            set $m [menu $M.m$m -tearoff 1 -postcommand [list gui::APLFiles $M.m$m]]
            set Removal [menu $M.m$m.remove -postcommand "[list gui::APLFilesRemove $M.m$m.remove]" ]
         }
         default {
            set $m [menu $M.m$m]
         }
      }
      $M add cascade -label $m -menu $M.m$m -underline $u
   }
   ##
   # Here is how to add a menu to the "system" menu
   # Use it to put "pinned" command
   #
   ontop::pinOnSystemMenu $M $::SA
   # FILE

   bindings::addCommand $File "Open File" {OpenFile} 0
   bindings::addCommand $File "Open Folder" {OpenFolder} 0
   bindings::addCommand $File "Open URL" {OpenURL}
   bindings::addCommand $File "Save Track As ..." {catalog::saveAs}
   $File add separator
   bindings::addCommand $File "Manage Media Library..."  {catalog::manageMediaFolders} 0
   bindings::addCommand $File "Manage Portable Player Library..."  {catalog::managePlayerFolders} 0
   bindings::addCommand $File "Index Media Library" {catalog::IndexAskFirst} 0
   $File add separator
   bindings::addCommand $File "Manage Media Folders to Monitor for Changes..." {
      package require saSettings
      folderManager::launch IncomingDirs
   }
   bindings::addCommand $File "Create Music Listing (csv files)" {db::document [file join $::HOME "listing.csv"]}
   $File add separator
   bindings::addCommand $File "Track Info..." {catalog::properties [list [soundControl::fileName]]}
   $File add separator
   bindings::addCommand $File "Exit" {taExit} 1

   # EDIT
   bindings::addCommand $Edit "Select All" {LB::setSelection 0 end;focus $LB} 0
   $Edit add separator
   bindings::addCommand $Edit "Sync APL to Selected Track" {SelectFile 0}
   bindings::addCommand $Edit "Sync APL to Playing Track" {SelectFile 1}
   bindings::addCommand $Edit "Locate Playing Track" {locatePlaying}
   $Edit add separator
   bindings::addCommand $Edit "Edit ID3 for Selected Track" {EditPLID3}
   bindings::addCommand $Edit "Edit ID3 for Playing Track" {EditPlaying} 0
   $Edit add separator
   bindings::addCommand $Edit "Rename Selected Track" {RenameFileDialog [LB::nameActive]}
   bindings::addCommand $Edit "Rename Playing Track" {RenameFileDialog [soundControl::fileName]} 0
   $Edit add separator
   # Need to figure out auto-binding for events like this
   bindings::addCommand $Edit "Copy Selected to Clipboard" {clipboardCopy $::LB 0} 0
   bindings::addCommand $Edit "Cut Selected to Clipboard" {clipboardCut $::LB 0} 1
   bindings::addCommand $Edit "Cut Selected append to Clipboard" {clipboardCut $::LB 1} 1
   bindings::addCommand $Edit "Paste Clipboard" {clipboardPaste $::LB 1} 0
   $Edit add separator
#   $Edit add command -label "Reset Playing Track Auto Gain"  -command {
#      gainControl::resetSome [list [soundControl::fileName]]
#      snack::audio scaling 1.0
#   }
   bindings::addCommand $Edit "Delete Selection" {Delete}


   bindings::addCommand $AutoPlaylists "Manage AutoPlaylists" {AutoPlaylist} 0
#   menu $AutoPlaylists.subsort
   # Below is also deleted by callback due to tearoff
#   gui::aplSubsortMenu $AutoPlaylists
   $AutoPlaylists add cascade -label "Remove..." -menu $Removal -underline 0  -compound left -image saImages::delete
   $AutoPlaylists add separator

   # SETTINGS
   if {$snackAmpSettings(enableConsole)} {
      $Tools add check -label "Show Console" -variable TA(Console) -underline 5 -command {
         if {[info command console] == {}} {
            catch {package require saConsole}
         }
         Console
      }
   }
   $Tools add check -label "Show Mini Player" -underline 5 -variable ::snackAmpSettings(activeMiniPlayer) -command {
      package require miniPlayer
      miniPlayer::toggle
   }
   bindings::addCommand $Tools "Show Mixer" {snack::mixerDialog} 2
   $Tools add separator
   if {[llength [snack::mixer lines]] > 1} {
      $Tools add cascade -label "Volume Controls..." -underline 0 -menu [menu $Tools.volume]
         foreach {line} [snack::mixer lines] {
            $Tools.volume add radiobutton -label $line -variable snackAmpSettings(lineSelect) -value $line
         }
   }
   # Force select the first (default) output device, fixes bug
   # on Linux where write errors occur randomly
   if {! [info exists ::snackAmpSettings(outputDevice)]} {
      set snackAmpSettings(outputDevice) [lindex [snack::audio outputDevices] 0]
   }
   snack::audio selectOutput $::snackAmpSettings(outputDevice)
   $Tools add cascade -label "Output Device..." -underline 0 -menu [menu $Tools.output]
      foreach {device} [snack::audio outputDevices] {
         $Tools.output add radiobutton -label $device -variable snackAmpSettings(outputDevice) -value $device \
            -command [list snack::audio selectOutput $::snackAmpSettings(outputDevice)]
      }
   $Tools add cascade -label "Minimum Bitrate..." -menu [menu $Tools.bitrate]
      set ::TA(minRate) 0
      foreach {rate} [list 0 24 64 96 128 140 160 180 192 256 320] {
         $Tools.bitrate add radiobutton -label $rate -variable ::TA(minRate) -value $rate
      }
   $Tools add cascade -label "Minimum Rating..." -menu [menu $Tools.rating]
      $Tools.rating add radiobutton -label "--" -variable ::snackAmpSettings(minTxtRating) -value "--"
      foreach {rate} [list 1 2 3 4 5] {
         set txt [string repeat * $rate]
         $Tools.rating add radiobutton -label $txt -variable ::snackAmpSettings(minTxtRating) -value $txt
      }
   if {[string length [info command ntSetPriority]] > 0} {
      $Tools add cascade -label "Runtime Priority..." -menu [menu $Tools.priority]
      ntSetPriority $::snackAmpSettings(priority)
      foreach {priority} [list high normal low] {
         $Tools.priority add radiobutton -label $priority \
            -variable ::snackAmpSettings(priority) -value $priority \
            -command {after 0 ntSetPriority $::snackAmpSettings(priority)}
      }
   }
   $Tools add separator
   #$Tools add check -label "Continuous Play" -variable snackAmpSettings(ContinuousPlay)
   #$Tools add check -label "Shuffle Play" -variable snackAmpSettings(randomPlay) \
   #   -command filtering::sortPlaylist
   #$Tools add separator
   bindings::addCommand $Tools "Manage Media Library..." {catalog::manageMediaFolders}
   bindings::addCommand $Tools "Manage Folders to Monitor for Changes..." {
      package require saSettings
      folderManager::launch IncomingDirs
   }
   $Tools add separator
   bindings::addCommand $Tools "Preferences..." {package require saSettings;settings::tabbedSettingsGui} 6

   # HELP
#   if {$::snackAmpSettings(EnableWebServer)==1} {
      bindings::addCommand $Help "User Manual" {::browser::help} 0
      bindings::addCommand $Help "Internal Web Server Home" {::browser::urlOpen $snackAmpSettings(internalWebBase)/} 0
#   } else {
#      bindings::addCommand $Help "User Manual" {::browser::urlOpen "http://snackamp.sourceforge.net/user_manual/index.html"} 0
#   }
   bindings::addCommand $Help "SnackAmp Home Page at SourceForge" {::browser::urlOpen "http://snackamp.sourceforge.net/"} 0
   $Help add separator
   #bindings::addCommand $Help "Versions"  gui::cvsVersions
   bindings::addCommand $Help "Check for Updates"  gui::updates
   bindings::addCommand $Help "About"  gui::about
}
#------------------------------------------------------------------------------
# Function   : InstallLBMenu
# Description: Install Listbox Menus and hard coded bindings
# Author     : Tom Wilkason
# Date       : 6/18/2002
#------------------------------------------------------------------------------
proc gui::InstallLBMenu {LB} {
   set m $LB.popup
   bindings::init $::SA menuBindings ;# Top Level Keys affect the playlist
   bindings::init $LB menuBindings   ;# Playlist unique keys override
   menu $m
   $m add cascade -label "Playlist Tracks" -menu $::TA(plPopUp)
   $m add cascade -label "Track(s) Rating" -menu $m.rating
   $m add separator
   bindings::addCommand $m "Play Selection" {Play}
   $m add separator
   bindings::addCommand $m "Copy" {clipboardCopy $::LB 0}
   bindings::addCommand $m "Cut" {clipboardCut $::LB 0}
   bindings::addCommand $m "Paste" {clipboardPaste $::LB 1}
   #bindings::addCommand $m "Stage" {linkToStaging $::LB 1}
   $m add separator
   bindings::addCommand $m "Sort Selection" {sortSelection}
   bindings::addCommand $m "Shuffle Selection" {shuffleSelection}
   $m add separator
   bindings::addCommand $m "Remove Selection" {Cut 1}
   bindings::addCommand $m "Clear Entire List" {LB::deleteAll;gui::updateFileDisplay}
   $m add separator
   bindings::addCommand $m "Delete File(s)" {Delete}
   bindings::addCommand $m "Rename File" {RenameFile [LB::nameActive]}
   $m add separator
   bindings::addCommand $m "Edit ID3 Info" {EditPLID3}
   bindings::addCommand $m "ID3 Table Editor" {PLNameToTag 0}
   bindings::addCommand $m "ID3 Tag To Name" {PLTagToName}
   $m add separator
   bindings::addCommand $m "Reset Auto Level" {gainControl::resetSome}
   menu $m.rating -tearoff 1 -title "Playlist Ratings"
   foreach {n} $db::ratingOptions {
      $m.rating add command -label [mrating $n] \
      -command [list LB::setSelectedRating $n]
   }
   $m add separator
   bindings::addCommand $m "Track Info..." {catalog::properties [LB::nameSelected]}

   bind [LB::body] <ButtonPress-3> {
      tk_popup $LB.popup %X %Y
   }
   # Bindings
   balloon::plDefine
   bind [LB::body] <Double-ButtonPress-1> {
      set here [$LB nearest [LB::Y %y]]
      LB::activate $here
      Play active
   }

   bind [LB::body] <B1-Motion> {
      LB::Drag [LB::Y %y]
      break
   }
   bind [LB::body] <ButtonPress-1> {
      focus %W
      LB::Select [LB::Y %y]
   }
}
#------------------------------------------------------------------------------
# Function   : updateTime
# Description: When Slider is updated, update time display, this is called
#              automatically when the time slider is updated during play.
# Author     : Tom Wilkason
# Date       : 1/9/2001
#------------------------------------------------------------------------------
proc gui::updateTime {args} {
   global TA
   variable trackPosition
   set elapsed   [expr {round($trackPosition)}]
   set remaining [expr {round($gui::songDuration - $elapsed)}]
   switch -- $::snackAmpSettings(showDetail) {
      1 {
         if {$gui::songDuration < 3600} {
            set TA(timeStatus) "[clock format $elapsed -format %M:%S]/[clock format $remaining -format %M:%S]"
         } else {
            set TA(timeStatus) "[clock format $elapsed -format %T -gmt 1]/[clock format $remaining -format %T -gmt 1]"
         }
      }
      2 {
         if {$gui::songDuration < 3600} {
            set TA(timeStatus) "[clock format $elapsed -format %M:%S]\n[clock format $remaining -format %M:%S]"
         } else {
            set TA(timeStatus) "[clock format $elapsed -format %T -gmt 1]\n[clock format $remaining -format %T -gmt 1]"
         }
      }
      default {
         if {$gui::songDuration < 3600} {
            set TA(timeStatus) "[clock format $elapsed -format %M:%S]\n[clock format $remaining -format %M:%S]\n[clock format [expr {round($gui::songDuration)}] -format %M:%S]"
         } else {
            set TA(timeStatus) "[clock format $elapsed -format %T -gmt 1]\n[clock format $remaining -format %T -gmt 1]\n[clock format [expr {round($gui::songDuration)}] -format %T -gmt 1]"
         }
      }
   }
}
#------------------------------------------------------------------------------
# Function   : showLevelMeter
# Description: Insert the level meter at the appropriate height
# Author     : Tom Wilkason
# Date       : 3/6/2003
#------------------------------------------------------------------------------
proc gui::showLevelMeter {args} {
   update
   set height [winfo reqheight $::SA.panel.stat]
   levelmeter::create $::SA.panel.stat.meter.lm -width 8 \
      -length $height -orient vertical -oncolor lightblue
   pack $::SA.panel.stat.meter.lm -side bottom -fill both -expand true
   after 1000 [list levelmeter::cmUpdate]
}

#------------------------------------------------------------------------------
# Function   : reLayoutPlayInfo
# Description: On a write to the showDetail variable, update the amount
#              of displayed info on the main gui
# Author     : Tom Wilkason
# Date       : 12/18/2003
#------------------------------------------------------------------------------
proc gui::reLayoutPlayInfo {hull sArray _G I mode} {
#   upvar #0 $_G G
#   set level $G($I)
   set level $::snackAmpSettings(showDetail) ;# called for more than showDetail trace
   foreach child [winfo children $hull] {
      destroy $child
   }
   gui::layoutPlayInfo $level $sArray $hull
   gui::fixMenuDetail $level
   updateTime
   wm geometry .saTop {}
}
##
# Place the top level menu in the appropriate place
#
proc gui::fixMenuDetail {level} {
   ##
   # Move menu bar to pop-up for very small layout
   #
   set tlm .saTop.bRmenubar ;# hack
   set tm $::SA.panel.stat.l.dtTitle ;# hack
   if {$level > 1} {
      if {[string length [$::SA cget -menu]]==0} {
         $::SA configure -menu $tlm
      }
      bind $tm <ButtonPress-3> {
         EditPlaying
      }
   } else {
      if {[string length [$::SA cget -menu]]!=0} {
         $::SA config -menu {}
      }
      bind $tm <ButtonPress-3> {
         tk_popup .saTop.bRmenubar %X %Y
      }
   }
}
#------------------------------------------------------------------------------
# Function   : gui::layoutPlayInfo
# Description: Layout the contents of the main window
# Author     : Tom Wilkason
# Date       : 1/12/2002
#------------------------------------------------------------------------------
proc gui::layoutPlayInfo {threshold _Var hull} {
   # Label Variable Row Col ColSpan Font
   set color(songLabel) tfg
   set color(songArtist) afg
   set color(songInfo) fg
   # Layout the data
   # 6 columns
   set gridData {
      1 Title  Title  0 5 songLabel
      2 Artist Artist 1 1 songArtist
      2 Rating Rating 1 1 songInfo
      2 Album  Album  2 1 songInfo
      2 Rate   Rate   2 1 songInfo
      3 Track  Track  3 1 songInfo
      3 Year   Year   3 1 songInfo
      3 Genre  Genre  3 1 songInfo
      4 Date   fDate  4 1 songInfo
      4 Size   fSize  4 1 songInfo
      4 Freq   sampRate 4 1 songInfo
   }
   set uCol 0
   set uRow 0
   ##
   # There is a tk bug when a full col is skipped
   #
   foreach {Use Title Field sRow cSpan font} $gridData {
      # label
      if {$threshold >= $Use} {
         # Special handling to work around tk col bug
         if {($Title=="Album" || $Title=="Artist") && $threshold>= 3} {
            incr cSpan 2
         }
         if {$sRow > $uRow} {
            set uCol 0
            set uRow $sRow
         }
         #W: Todo, link the label to a variable which can be cleared
         # if there is no value. $Title
         if {$::snackAmpSettings(showLabels)} {
            set txt $Title
         } else {
            set txt " "
         }
         label $hull.lb$Title -text $txt  -font smallLabel -pady 0 \
            -background $::snackAmpSettings(bg) \
            -foreground $::snackAmpSettings(lfg) \
            -anchor e
         settings::regColor $hull.lb$Title \
            -background ::snackAmpSettings(bg) \
            -foreground ::snackAmpSettings(lfg)
         label $hull.dt$Title -textvariable ${_Var}($Field) -font $font -pady 0  -bd 0 \
            -background $::snackAmpSettings(bg) \
            -foreground $::snackAmpSettings($color($font)) \
            -anchor w -bd 0
         settings::regColor $hull.dt$Title \
            -background ::snackAmpSettings(bg) \
            -foreground ::snackAmpSettings($color($font))
            if {$Title=="Rating"} {
               destroy $hull.rm
               set pm $hull.rm
               menu $pm -postcommand [list gui::ratingPopup $pm]
               balloon::define $hull.dt$Title "Adjust rating of the playing track, the more stars the better."
               bind $hull.dt$Title <ButtonPress-1> [list tk_popup $pm %X %Y]
            }
         # If only title, then don't bother with label
         # No label on minimal threshold
         if {$threshold > 1} {
            grid $hull.lb$Title -row $sRow -column $uCol -sticky e -pady 0
         }
         grid $hull.dt$Title -row $sRow -column [incr uCol] -columnspan $cSpan -sticky w -pady 0
         incr uCol $cSpan
      }
   }
   grid columnconfigure $hull 1 -weight 1
}
#------------------------------------------------------------------------------
# Function   : gui::playListFiles
# Description: postcommand callback to show play list files on menu, called
#              from play list menu postcommand
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc gui::playListFiles {menu} {
   global TA LB
   set Files [catalog::getPlaylists]
   if {[llength $Files] == 0} {
      return
   }
   $menu.mAppend delete 0 end
   $menu.mReplace delete 0 end
   $menu.mInsert delete 0 end
   $menu.uAppend delete 0 end
   $menu.uReplace delete 0 end
   ##
   # Need to sort the list by decending date
   #
   foreach {File} $Files {
      set Value [file rootname [file tail $File]]
      $menu.mAppend add radiobutton -label $Value -value $Value -variable ::TA(PlayListSelection) \
         -command [list macros::ReadPlaylistFile $File "append"]
      $menu.mInsert add radiobutton -label $Value -value $Value -variable ::TA(PlayListSelection) \
         -command [list macros::ReadPlaylistFile $File "insert"]
      $menu.mReplace add radiobutton -label $Value -value $Value -variable ::TA(PlayListSelection) \
         -command [list macros::ReadPlaylistFile $File "replace"]

      $menu.uAppend add radiobutton -label $Value -value $Value -variable ::TA(PlayListSelection) \
         -command [list macros::SavePlaylistFile $::LB "append" "selected" $File]
      $menu.uReplace add radiobutton -label $Value -value $Value -variable ::TA(PlayListSelection) \
         -command [list macros::SavePlaylistFile $::LB "replace" "selected" $File]
   }
}

#------------------------------------------------------------------------------
# Pop-up to show active media players
#------------------------------------------------------------------------------
proc gui::mediaPlayerPopup {menu command} {
   $menu delete 0 end
   foreach {player} $::snackAmpSettings(mp3Player) {
      if {[catalog::isMountedFolder $player]} {
         $menu add command -label "$player" \
         -command [list macros::copyToMediaPlayer $player $command] \
         -compound left -image saImages::mp3_player.close
      }
   }
}
#------------------------------------------------------------------------------
# Popup menu to adjust curent file stuff
#------------------------------------------------------------------------------
proc gui::ratingPopup {menu} {
   global TA
   variable trackRating
   $menu delete 0 end
   set me [soundControl::fileName]
   set trackRating [mrating [db::getValues $me Rating]]
   foreach {n} $db::ratingOptions {
      $menu add radiobutton -label [mrating $n] \
      -value [mrating $n] \
      -variable gui::trackRating \
      -command [list gui::setTrackRating $me $n]
   }
}

#------------------------------------------------------------------------------
# Set the current tracks rating and display info
#------------------------------------------------------------------------------
proc gui::setTrackRating {track val} {
   db::setVal $track Rating $val
   ;# TFW: Can we use fancy stars instead of ***
   set ::songArray(Rating) [mrating $val]
}
#------------------------------------------------------------------------------
# Function   : gui::APLFiles
# Description: post command Callback to show auto playlist setting files, called
#              from AutoPlaylist menu postcommand
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc gui::APLFiles {menu} {
   global TA LB
   set Files [catalog::getAutoPlaylists]
   if {[llength $Files] == 0} {
      return
   }
   $menu delete 3 end
   $menu add separator
   foreach {File} $Files {
      set Value [file rootname [file tail $File]]
      $menu add radiobutton -label $Value -value $Value \
         -variable TA(AutoPlayListSelection) -command [list readAPLFile $File 1]
   }
}
#------------------------------------------------------------------------------
# Function   : gui::APLFilesRemove
# Description: post command Callback to allow removal of auto playlist setting files
#              from AutoPlaylist menu postcommand
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc gui::APLFilesRemove {menu} {
   global TA LB
   set Files [catalog::getAutoPlaylists]
   if {[llength $Files] == 0} {
      return
   }
   $menu delete 0 end
   foreach {File} $Files {
      set Value [file rootname [file tail $File]]
      $menu add radiobutton -label $Value -value $Value \
         -command [list removeAPLFile $File]
   }
}

#------------------------------------------------------------------------------
# Function   : gui::PLFilesRemove
# Description: post command Callback to allow removal of auto playlist setting files
#              from AutoPlaylist menu postcommand
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc gui::PLFilesRemove {menu} {
   global TA LB
   set Files [catalog::getPlaylists]
   if {[llength $Files]==0} {
      return
   }
   $menu delete 0 end
   foreach {File} $Files {
      set Value [file rootname [file tail $File]]
      $menu add radiobutton -label $Value -value $Value \
         -command [list removePLFile $File]
   }
}
#------------------------------------------------------------------------------
# Function   : about
# Description: About dialog box
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc gui::snackVersion {args} {
   set results "Snack engine version $::snack::patchLevel"
   if {[catch {package present snackogg} result]} {
      append results "\nSnack ogg library not present"
   } else {
      append results "\nSnack ogg version $result"
   }
   return $results
}
proc gui::pkgVersion {pkg} {
   if {[catch {package present $pkg} result]} {
      set results "$pkg library not present"
   } else {
      set results "$pkg version $result"
   }
   return $results
}
proc gui::about {args} {
Wait 1
set Message {SnackAmp Music Player version $::SAVersion
[gui::snackVersion]
[gui::pkgVersion tkdnd]
[gui::pkgVersion Img]
Tcl/Tk version [set ::tcl_patchLevel]

Snackamp user folder located at:
$::HOME

Using $::TA(database) database
Current Database Statistics
[db::statsText "   "]

Features:
- Plays wav, mp3 and ogg files and http streams
- Complete playlist, media and tag mangement
- Integrated web server for remote control and serving streaming music

 2002-2005 Tom Wilkason - enjoy
  tom.wilkason@cox.net}
   clipboard clear
   clipboard append [subst $Message]

   # Improved dialog
   set W .about
   destroy $W
   set splash [toplevel $W -class Dialog]
   wm withdraw $W
   catch {wm attributes $W -toolwindow 1}
   wm title $W "SnackAmp $::SAVersion"
   frame $W.f -padx 5 -pady 10
   label $W.f.lab -image saImages::saLogo
   message $W.msg -text [subst $Message] -justify left -anchor w
   pack $W.f.lab -padx 10 -pady 20
   grid $W.f $W.msg -sticky n

   set link [button $W.link -text "http://snackamp.sourceforge.net/" \
   -command {::browser::urlOpen "http://snackamp.sourceforge.net/"} -relief flat -fg blue]
   grid x $link -sticky w

   grid [horzLine $W.l] -sticky new -columnspan 2 -pady 4

   set stop [button $W.stop -text "OK" -default active -relief raised -compound left -image saImages::Apply \
      -pady 2 -padx 4 -command [list destroy $W]]
   grid $stop -columnspan 2

   bind Dialog <Return> [list $W.stop invoke]
   # center it
   centerWindow $W
   wm deiconify $W
   raise $W
   Wait 0
}
##
# Dialog showing the CVS Headers
#
proc gui::cvsVersions {args} {
   append vinfo "CVS Version Info:\n"
   foreach {_module} [lsort -dictionary [info globals *_History]] {
      global $_module
      upvar $_module module
      foreach {xx file rest} [split $module ":,"] {break}
      foreach {v ver date time user xx yy} $rest {break}
      append vinfo [format "%-5s\t%-10s\t%-60s" $ver $date [file tail $file]]\n
   }
   clipboard clear
   clipboard append $vinfo
   tk_messageBox -title "OK to Continue" -icon info -message $vinfo
}
proc gui::textVersions {args} {
   # Read internet version file
   set vinfo ""
   if {[gui::versions vdata]} {
      append vinfo "\nBuild Versions\n"
      foreach {key value link} $vdata {
         append vinfo [format "\n%-25s\t%-10s" $key $value]
      }
   }
   return $vinfo
}
proc gui::versions {_vdata} {
   upvar $_vdata vdata
   # Read locale file
   set vdata [list]
   set lvf [file join $::TA(snackRoot) "myversion.txt"]
   if {[catch {set fid [open $lvf r]} error] } then {
      lappend vdata "This Version Info" "!Could not read myversion.txt" ""
   } else {
      foreach token [read $fid] {
         lappend vdata $token
      }
      close $fid
   }
   set ok 0
   if {[catch {macros::getUrlBinary "http://snackamp.sourceforge.net/version.txt"} contents] } then {
      saLog $contents
   } elseif {[string match "*Latest*" $contents]} {
      foreach token $contents {
         lappend vdata $token
      }
      incr ok
   }
   return $ok
}
#----------------------------------------------------------------------------
# Show version info and check for updates
#----------------------------------------------------------------------------
proc gui::updates {args} {
   # Improved dialog
   if {[gui::versions vdata]==0} {
      tk_messageBox -title "Error" -icon warning \
         -message "Failed to retrieve version info from sourceforge\nVerify your proxy and/or internet connection"
      return
   }
   set W .about
   destroy $W
   set splash [toplevel $W -class Dialog]
   wm withdraw $W
   catch {wm attributes $W -toolwindow 1}
   wm title $W "SnackAmp $::SAVersion"
   frame $W.f -padx 5 -pady 10
   label $W.f.lab -image saImages::saLogo
   label $W.msg -text "Version Information Below" -justify left -anchor w
   pack $W.f.lab -padx 10 -pady 5
   grid $W.f $W.msg
   set i 0
   foreach {title date link} $vdata {
      set widgets [list]
      lappend widgets [label $W.[incr i] -text $title] \
         [button $W.[incr i] -text $date -relief flat -fg blue \
            -command [list ::browser::urlOpen $link]]
      balloon::define $W.$i "Launch browser on $link"
      eval grid $widgets -sticky w
   }
   grid [horzLine $W.l] -sticky new -columnspan 2 -pady 4

   set stop [button $W.stop -text "OK" -default active -relief raised -compound left -image saImages::Apply \
      -pady 2 -padx 4 -command [list destroy $W]]
   grid $stop -columnspan 2

   bind Dialog <Return> [list $W.stop invoke]
   # center it
   centerWindow $W
   wm deiconify $W
   raise $W
   Wait 0
}

##
# Update the number of files in the playlist
# Note: Fix when we commonize sortMode
proc gui::updateFileDisplay {} {
   Trace
   global snackAmpSettings LB TA
   array set lookup {
      0  "Sorted"
      1  "Shuffled"
      2  "Shuffled Folders, Sorted Tracks"
      3  "Shuffled Folders, Shuffled Tracks"
   }
   set mode "[commify [LB::idxEnd]] Tracks ($snackAmpSettings(sortModeTxt)) [LB::totTime]"
   $::SA.hull.label configure -text $mode
}
#----------------------------------------------------------------------------
# Allow mousewheel to adjust the overall font sizing
#----------------------------------------------------------------------------
proc gui::hookFontSizeAdjust {args} {
   if {$::tcl_platform(platform) != "windows"} {
      bind $::SA <Control-Button-5> [list gui::fontSizeAdjust 0.05]
      bind $::SA <Control-Button-4> [list gui::fontSizeAdjust -0.05]
   } else {
      bind $::SA <Control-MouseWheel> {
         set amount [expr {%D/abs(%D)*(-0.05)}] ;# get +/- 2
         gui::fontSizeAdjust $amount
      }
   }

}
proc gui::fontSizeAdjust {amount} {
   global snackAmpSettings
   set snackAmpSettings(FontSizeFactor) [expr {$snackAmpSettings(FontSizeFactor)+$amount}]
   settings::saFontSize $snackAmpSettings(FontSizeFactor)
}

package provide saGUI 1.0
